# Copyright (c) 2016 Michael Gruener <michael.gruener@chaosmoon.net>
# Copyright (c) 2021 Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import abc
import typing as t

from ansible_collections.community.crypto.plugins.module_utils._acme.errors import (
    ModuleFailException,
)
from ansible_collections.community.crypto.plugins.module_utils._acme.utils import (
    der_to_pem,
    process_links,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.pem import (
    split_pem_list,
)

if t.TYPE_CHECKING:
    from ansible_collections.community.crypto.plugins.module_utils._acme.acme import (  # pragma: no cover
        ACMEClient,
    )


_CertificateChain = t.TypeVar("_CertificateChain", bound="CertificateChain")


class CertificateChain:
    """
    Download and parse the certificate chain.
    https://tools.ietf.org/html/rfc8555#section-7.4.2
    """

    def __init__(self, url: str):
        self.url = url
        self.cert: str | None = None
        self.chain: list[str] = []
        self.alternates: list[str] = []

    @classmethod
    def download(
        cls: type[_CertificateChain], *, client: ACMEClient, url: str
    ) -> _CertificateChain:
        content, info = client.get_request(
            url,
            parse_json_result=False,
            headers={"Accept": "application/pem-certificate-chain"},
        )

        if not content or not info["content-type"].startswith(
            "application/pem-certificate-chain"
        ):
            raise ModuleFailException(
                f"Cannot download certificate chain from {url}, as content type is not application/pem-certificate-chain: {content!r} (headers: {info})"
            )

        result = cls(url)

        # Parse data
        certs = split_pem_list(content.decode("utf-8"), keep_inbetween=True)
        if certs:
            result.cert = certs[0]
            result.chain = certs[1:]

        process_links(
            info=info,
            callback=lambda link, relation: result._process_links(  # pylint: disable=protected-access
                client=client, link=link, relation=relation
            ),
        )

        if result.cert is None:
            raise ModuleFailException(
                f"Failed to parse certificate chain download from {url}: {content!r} (headers: {info})"
            )

        return result

    def _process_links(self, *, client: ACMEClient, link: str, relation: str) -> None:
        if relation == "up":
            # Process link-up headers if there was no chain in reply
            if not self.chain:
                chain_result, chain_info = client.get_request(
                    link, parse_json_result=False
                )
                if chain_info["status"] in [200, 201]:
                    self.chain.append(der_to_pem(chain_result))
        elif relation == "alternate":
            self.alternates.append(link)

    def to_json(self) -> dict[str, bytes]:
        if self.cert is None:
            raise ValueError("Has no certificate")
        cert = self.cert.encode("utf8")
        chain = ("\n".join(self.chain)).encode("utf8")
        return {
            "cert": cert,
            "chain": chain,
            "full_chain": cert + chain,
        }


class Criterium:
    def __init__(self, *, criterium: dict[str, t.Any], index: int):
        self.index = index
        self.test_certificates: t.Literal["first", "last", "all"] = criterium[
            "test_certificates"
        ]
        self.subject: dict[str, t.Any] | None = criterium["subject"]
        self.issuer: dict[str, t.Any] | None = criterium["issuer"]
        self.subject_key_identifier: str | None = criterium["subject_key_identifier"]
        self.authority_key_identifier: str | None = criterium[
            "authority_key_identifier"
        ]


class ChainMatcher(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def match(self, *, certificate: CertificateChain) -> bool:
        """
        Check whether a certificate chain (CertificateChain instance) matches.
        """


__all__ = ("CertificateChain", "ChainMatcher", "Criterium")



#!/usr/bin/python
# Copyright (c) 2016-2017, Yanis Guenane <yanis+ansible@guenane.org>
# Copyright (c) 2017, Markus Teufelberger <mteufelberger+ansible@mgit.at>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: openssl_csr_info
short_description: Provide information of OpenSSL Certificate Signing Requests (CSR)
description:
  - This module allows one to query information on OpenSSL Certificate Signing Requests (CSR).
  - In case the CSR signature cannot be validated, the module will fail. In this case, all return variables are still returned.
  - It uses the cryptography python library to interact with OpenSSL.
author:
  - Felix Fontein (@felixfontein)
  - Yanis Guenane (@Spredzy)
extends_documentation_fragment:
  - community.crypto._attributes
  - community.crypto._attributes.info_module
  - community.crypto._attributes.idempotent_not_modify_state
  - community.crypto._cryptography_dep.minimum
  - community.crypto._name_encoding
options:
  path:
    description:
      - Remote absolute path where the CSR file is loaded from.
      - Either O(path) or O(content) must be specified, but not both.
    type: path
  content:
    description:
      - Content of the CSR file.
      - Either O(path) or O(content) must be specified, but not both.
    type: str
    version_added: "1.0.0"
  select_crypto_backend:
    description:
      - Determines which crypto backend to use.
      - The default choice is V(auto), which tries to use C(cryptography) if available.
      - If set to V(cryptography), will try to use the L(cryptography,https://cryptography.io/) library.
      - Note that with community.crypto 3.0.0, all values behave the same.
        This option will be deprecated in a later version.
        We recommend to not set it explicitly.
    type: str
    default: auto
    choices: [auto, cryptography]

seealso:
  - module: community.crypto.openssl_csr
  - module: community.crypto.openssl_csr_pipe
  - plugin: community.crypto.openssl_csr_info
    plugin_type: filter
    description: A filter variant of this module.
  - plugin: community.crypto.to_serial
    plugin_type: filter
"""

EXAMPLES = r"""
---
- name: Generate an OpenSSL Certificate Signing Request
  community.crypto.openssl_csr:
    path: /etc/ssl/csr/www.ansible.com.csr
    privatekey_path: /etc/ssl/private/ansible.com.pem
    common_name: www.ansible.com

- name: Get information on the CSR
  community.crypto.openssl_csr_info:
    path: /etc/ssl/csr/www.ansible.com.csr
  register: result

- name: Dump information
  ansible.builtin.debug:
    var: result
"""

RETURN = r"""
signature_valid:
  description:
    - Whether the CSR's signature is valid.
    - In case the check returns V(false), the module will fail.
  returned: success
  type: bool
basic_constraints:
  description: Entries in the C(basic_constraints) extension, or V(none) if extension is not present.
  returned: success
  type: list
  elements: str
  sample: ['CA:TRUE', 'pathlen:1']
basic_constraints_critical:
  description: Whether the C(basic_constraints) extension is critical.
  returned: success
  type: bool
extended_key_usage:
  description: Entries in the C(extended_key_usage) extension, or V(none) if extension is not present.
  returned: success
  type: list
  elements: str
  sample: [Biometric Info, DVCS, Time Stamping]
extended_key_usage_critical:
  description: Whether the C(extended_key_usage) extension is critical.
  returned: success
  type: bool
extensions_by_oid:
  description: Returns a dictionary for every extension OID.
  returned: success
  type: dict
  contains:
    critical:
      description: Whether the extension is critical.
      returned: success
      type: bool
    value:
      description:
        - The Base64 encoded value (in DER format) of the extension.
        - B(Note) that depending on the C(cryptography) version used, it is not possible to extract the ASN.1 content of the
          extension, but only to provide the re-encoded content of the extension in case it was parsed by C(cryptography).
          This should usually result in exactly the same value, except if the original extension value was malformed.
      returned: success
      type: str
      sample: "MAMCAQU="
  sample: {"1.3.6.1.5.5.7.1.24": {"critical": false, "value": "MAMCAQU="}}
key_usage:
  description: Entries in the C(key_usage) extension, or V(none) if extension is not present.
  returned: success
  type: str
  sample: [Key Agreement, Data Encipherment]
key_usage_critical:
  description: Whether the C(key_usage) extension is critical.
  returned: success
  type: bool
subject_alt_name:
  description:
    - Entries in the C(subject_alt_name) extension, or V(none) if extension is not present.
    - See O(name_encoding) for how IDNs are handled.
  returned: success
  type: list
  elements: str
  sample: ["DNS:www.ansible.com", "IP:1.2.3.4"]
subject_alt_name_critical:
  description: Whether the C(subject_alt_name) extension is critical.
  returned: success
  type: bool
ocsp_must_staple:
  description: V(true) if the OCSP Must Staple extension is present, V(none) otherwise.
  returned: success
  type: bool
ocsp_must_staple_critical:
  description: Whether the C(ocsp_must_staple) extension is critical.
  returned: success
  type: bool
name_constraints_permitted:
  description: List of permitted subtrees to sign certificates for.
  returned: success
  type: list
  elements: str
  sample: ['email:.somedomain.com']
  version_added: 1.1.0
name_constraints_excluded:
  description:
    - List of excluded subtrees the CA cannot sign certificates for.
    - Is V(none) if extension is not present.
    - See O(name_encoding) for how IDNs are handled.
  returned: success
  type: list
  elements: str
  sample: ['email:.com']
  version_added: 1.1.0
name_constraints_critical:
  description:
    - Whether the C(name_constraints) extension is critical.
    - Is V(none) if extension is not present.
  returned: success
  type: bool
  version_added: 1.1.0
subject:
  description:
    - The CSR's subject as a dictionary.
    - Note that for repeated values, only the last one will be returned.
  returned: success
  type: dict
  sample: {"commonName": "www.example.com", "emailAddress": "test@example.com"}
subject_ordered:
  description: The CSR's subject as an ordered list of tuples.
  returned: success
  type: list
  elements: list
  sample: [["commonName", "www.example.com"], ["emailAddress": "test@example.com"]]
public_key:
  description: CSR's public key in PEM format.
  returned: success
  type: str
  sample: "-----BEGIN PUBLIC KEY-----\nMIICIjANBgkqhkiG9w0BAQEFAAOCAg8A..."
public_key_type:
  description:
    - The CSR's public key's type.
    - One of V(RSA), V(DSA), V(ECC), V(Ed25519), V(X25519), V(Ed448), or V(X448).
    - Will start with V(unknown) if the key type cannot be determined.
  returned: success
  type: str
  version_added: 1.7.0
  sample: RSA
public_key_data:
  description:
    - Public key data. Depends on the public key's type.
  returned: success
  type: dict
  version_added: 1.7.0
  contains:
    size:
      description:
        - Bit size of modulus (RSA) or prime number (DSA).
      type: int
      returned: When RV(public_key_type=RSA) or RV(public_key_type=DSA)
    modulus:
      description:
        - The RSA key's modulus.
      type: int
      returned: When RV(public_key_type=RSA)
    exponent:
      description:
        - The RSA key's public exponent.
      type: int
      returned: When RV(public_key_type=RSA)
    p:
      description:
        - The C(p) value for DSA.
        - This is the prime modulus upon which arithmetic takes place.
      type: int
      returned: When RV(public_key_type=DSA)
    q:
      description:
        - The C(q) value for DSA.
        - This is a prime that divides C(p - 1), and at the same time the order of the subgroup of the multiplicative group
          of the prime field used.
      type: int
      returned: When RV(public_key_type=DSA)
    g:
      description:
        - The C(g) value for DSA.
        - This is the element spanning the subgroup of the multiplicative group of the prime field used.
      type: int
      returned: When RV(public_key_type=DSA)
    curve:
      description:
        - The curve's name for ECC.
      type: str
      returned: When RV(public_key_type=ECC)
    exponent_size:
      description:
        - The maximum number of bits of a private key. This is basically the bit size of the subgroup used.
      type: int
      returned: When RV(public_key_type=ECC)
    x:
      description:
        - The C(x) coordinate for the public point on the elliptic curve.
      type: int
      returned: When RV(public_key_type=ECC)
    y:
      description:
        - For RV(public_key_type=ECC), this is the C(y) coordinate for the public point on the elliptic curve.
        - For RV(public_key_type=DSA), this is the publicly known group element whose discrete logarithm w.r.t. C(g) is the
          private key.
      type: int
      returned: When RV(public_key_type=DSA) or RV(public_key_type=ECC)
public_key_fingerprints:
  description:
    - Fingerprints of CSR's public key.
    - For every hash algorithm available, the fingerprint is computed.
  returned: success
  type: dict
  sample: "{'sha256': 'd4:b3:aa:6d:c8:04:ce:4e:ba:f6:29:4d:92:a3:94:b0:c2:ff:bd:bf:33:63:11:43:34:0f:51:b0:95:09:2f:63', 'sha512':
    'f7:07:4a:f0:b0:f0:e6:8b:95:5f:f9:e6:61:0a:32:68:f1..."
subject_key_identifier:
  description:
    - The CSR's subject key identifier.
    - The identifier is returned in hexadecimal, with V(:) used to separate bytes.
    - Is V(none) if the C(SubjectKeyIdentifier) extension is not present.
  returned: success
  type: str
  sample: '00:11:22:33:44:55:66:77:88:99:aa:bb:cc:dd:ee:ff:00:11:22:33'
authority_key_identifier:
  description:
    - The CSR's authority key identifier.
    - The identifier is returned in hexadecimal, with V(:) used to separate bytes.
    - Is V(none) if the C(AuthorityKeyIdentifier) extension is not present.
  returned: success
  type: str
  sample: '00:11:22:33:44:55:66:77:88:99:aa:bb:cc:dd:ee:ff:00:11:22:33'
authority_cert_issuer:
  description:
    - The CSR's authority cert issuer as a list of general names.
    - Is V(none) if the C(AuthorityKeyIdentifier) extension is not present.
    - See O(name_encoding) for how IDNs are handled.
  returned: success
  type: list
  elements: str
  sample: ["DNS:www.ansible.com", "IP:1.2.3.4"]
authority_cert_serial_number:
  description:
    - The CSR's authority cert serial number.
    - Is V(none) if the C(AuthorityKeyIdentifier) extension is not present.
    - This return value is an B(integer). If you need the serial numbers as a colon-separated hex string, such as C(11:22:33),
      you need to convert it to that form with P(community.crypto.to_serial#filter).
  returned: success
  type: int
  sample: 12345
"""

import typing as t

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.crypto.plugins.module_utils._crypto.basic import (
    OpenSSLObjectError,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.module_backends.csr_info import (
    select_backend,
)


def main() -> t.NoReturn:
    module = AnsibleModule(
        argument_spec={
            "path": {"type": "path"},
            "content": {"type": "str"},
            "name_encoding": {
                "type": "str",
                "default": "ignore",
                "choices": ["ignore", "idna", "unicode"],
            },
            "select_crypto_backend": {
                "type": "str",
                "default": "auto",
                "choices": ["auto", "cryptography"],
            },
        },
        required_one_of=(["path", "content"],),
        mutually_exclusive=(["path", "content"],),
        supports_check_mode=True,
    )

    content: str | None = module.params["content"]
    path: str | None = module.params["path"]
    if content is not None:
        data = content.encode("utf-8")
    else:
        if path is None:
            module.fail_json(msg="One of content and path must be provided")
        try:
            with open(path, "rb") as f:
                data = f.read()
        except (IOError, OSError) as e:
            module.fail_json(msg=f"Error while reading CSR file from disk: {e}")

    module_backend = select_backend(
        module=module, content=data, validate_signature=True
    )

    try:
        result = module_backend.get_info()
        module.exit_json(**result)
    except OpenSSLObjectError as exc:
        module.fail_json(msg=str(exc))


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_wireless_ethernet_ports_profiles
short_description: Resource module for networks _wireless _ethernet _ports _profiles
description:
  - Manage operations update and delete of the resource networks _wireless _ethernet
    _ports _profiles.
  - Delete an AP port profile.
  - Update the AP port profile by ID for this network.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  name:
    description: AP port profile name.
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  ports:
    description: AP ports configuration.
    elements: dict
    suboptions:
      enabled:
        description: AP port enabled.
        type: bool
      name:
        description: AP port name.
        type: str
      pskGroupId:
        description: AP port PSK Group number.
        type: str
      ssid:
        description: AP port ssid number.
        type: int
    type: list
  profileId:
    description: ProfileId path parameter. Profile ID.
    type: str
  usbPorts:
    description: AP usb ports configuration.
    elements: dict
    suboptions:
      enabled:
        description: AP usb port enabled.
        type: bool
      name:
        description: AP usb port name.
        type: str
      ssid:
        description: AP usb port ssid number.
        type: int
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for wireless deleteNetworkWirelessEthernetPortsProfile
    description: Complete reference of the deleteNetworkWirelessEthernetPortsProfile
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!delete-network-wireless-ethernet-ports-profile
  - name: Cisco Meraki documentation for wireless updateNetworkWirelessEthernetPortsProfile
    description: Complete reference of the updateNetworkWirelessEthernetPortsProfile
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-wireless-ethernet-ports-profile
notes:
  - SDK Method used are
    wireless.Wireless.delete_network_wireless_ethernet_ports_profile,
    wireless.Wireless.update_network_wireless_ethernet_ports_profile,
  - Paths used are
    delete /networks/{networkId}/wireless/ethernet/ports/profiles/{profileId},
    put /networks/{networkId}/wireless/ethernet/ports/profiles/{profileId},
"""

EXAMPLES = r"""
- name: Update by id
  cisco.meraki.networks_wireless_ethernet_ports_profiles:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    name: string
    networkId: string
    ports:
      - enabled: true
        name: string
        pskGroupId: string
        ssid: 0
    profileId: string
    usbPorts:
      - enabled: true
        name: string
        ssid: 0
- name: Delete by id
  cisco.meraki.networks_wireless_ethernet_ports_profiles:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: absent
    networkId: string
    profileId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "profileId": "string",
      "name": "string",
      "isDefault": true,
      "ports": [
        {
          "name": "string",
          "number": 0,
          "enabled": true,
          "ssid": 0,
          "pskGroupId": "string"
        }
      ],
      "usbPorts": [
        {
          "name": "string",
          "enabled": true,
          "ssid": 0
        }
      ]
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: dynamodb_ttl
version_added: 1.0.0
short_description: Set TTL for a given DynamoDB table
description:
- Sets the TTL for a given DynamoDB table.
options:
  state:
    description:
    - State to set DynamoDB table to.
    choices: ['enable', 'disable']
    required: false
    type: str
  table_name:
    description:
    - Name of the DynamoDB table to work on.
    required: true
    type: str
  attribute_name:
    description:
    - The name of the Time To Live attribute used to store the expiration time for items in the table.
    - This appears to be required by the API even when disabling TTL.
    required: true
    type: str

author:
- Ted Timmons (@tedder)
extends_documentation_fragment:
- amazon.aws.common.modules
- amazon.aws.region.modules
- amazon.aws.boto3
"""

EXAMPLES = r"""
- name: enable TTL on my cowfacts table
  community.aws.dynamodb_ttl:
    state: enable
    table_name: cowfacts
    attribute_name: cow_deleted_date

- name: disable TTL on my cowfacts table
  community.aws.dynamodb_ttl:
    state: disable
    table_name: cowfacts
    attribute_name: cow_deleted_date
"""

RETURN = r"""
current_status:
  description: current or new TTL specification.
  type: dict
  returned: always
  sample:
  - { "AttributeName": "deploy_timestamp", "TimeToLiveStatus": "ENABLED" }
  - { "AttributeName": "deploy_timestamp", "Enabled": true }
"""

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def get_current_ttl_state(c, table_name):
    """Fetch the state dict for a table."""
    current_state = c.describe_time_to_live(TableName=table_name)
    return current_state.get("TimeToLiveDescription")


def does_state_need_changing(attribute_name, desired_state, current_spec):
    """Run checks to see if the table needs to be modified. Basically a dirty check."""
    if not current_spec:
        # we don't have an entry (or a table?)
        return True

    if desired_state.lower() == "enable" and current_spec.get("TimeToLiveStatus") not in ["ENABLING", "ENABLED"]:
        return True
    if desired_state.lower() == "disable" and current_spec.get("TimeToLiveStatus") not in ["DISABLING", "DISABLED"]:
        return True
    if attribute_name != current_spec.get("AttributeName"):
        return True

    return False


def set_ttl_state(c, table_name, state, attribute_name):
    """Set our specification. Returns the update_time_to_live specification dict,
    which is different than the describe_* call."""
    is_enabled = False
    if state.lower() == "enable":
        is_enabled = True

    ret = c.update_time_to_live(
        TableName=table_name,
        TimeToLiveSpecification={
            "Enabled": is_enabled,
            "AttributeName": attribute_name,
        },
    )

    return ret.get("TimeToLiveSpecification")


def main():
    argument_spec = dict(
        state=dict(choices=["enable", "disable"]),
        table_name=dict(required=True),
        attribute_name=dict(required=True),
    )
    module = AnsibleAWSModule(
        argument_spec=argument_spec,
    )

    try:
        dbclient = module.client("dynamodb")
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS")

    result = {"changed": False}

    # wrap all our calls to catch the standard exceptions. We don't pass `module` in to the
    # methods so it's easier to do here.
    try:
        current_state = get_current_ttl_state(dbclient, module.params["table_name"])

        if does_state_need_changing(module.params["attribute_name"], module.params["state"], current_state):
            # changes needed
            new_state = set_ttl_state(
                dbclient, module.params["table_name"], module.params["state"], module.params["attribute_name"]
            )
            result["current_status"] = new_state
            result["changed"] = True
        else:
            # no changes needed
            result["current_status"] = current_state

    except botocore.exceptions.ClientError as e:
        module.fail_json_aws(e, msg="Failed to get or update ttl state")
    except botocore.exceptions.ParamValidationError as e:
        module.fail_json_aws(e, msg="Failed due to invalid parameters")
    except ValueError as e:
        module.fail_json_aws(e, msg="Failed")

    module.exit_json(**result)


if __name__ == "__main__":
    main()



from __future__ import absolute_import, division, print_function


__metaclass__ = type
import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


def _tmplt_ospf_default_information(config_data):
    if "default_information_originate" in config_data:
        command = "default-information originate"
        if "always" in config_data["default_information_originate"]:
            command += " always"
        if "metric" in config_data["default_information_originate"]:
            command += " metric {metric}".format(**config_data["default_information_originate"])
        if "metric_type" in config_data["default_information_originate"]:
            command += " metric-type {metric_type}".format(
                **config_data["default_information_originate"],
            )
        if "route_policy" in config_data["default_information_originate"]:
            command += " route-policy {route_policy}".format(
                **config_data["default_information_originate"],
            )
        return command


def _tmplt_ospf_auto_cost(config_data):
    if "auto_cost" in config_data:
        command = "auto-cost"
        if "disable" in config_data["auto_cost"]:
            command += " disable"
        if "reference_bandwidth" in config_data["auto_cost"]:
            command += " reference-bandwidth {reference_bandwidth}".format(
                **config_data["auto_cost"],
            )
        return command


def _tmplt_ospf_bfd(config_data):
    if "bfd" in config_data:
        command = "bfd"
        if "minimum_interval" in config_data["bfd"]:
            command += " minimum-interval {minimum_interval}".format(**config_data["bfd"])

        if "multiplier" in config_data["bfd"]:
            command += " multiplier {multiplier}".format(**config_data["bfd"])

        return command


def _tmplt_ospf_security(config_data):
    if "security_ttl" in config_data:
        command = "security_ttl"
        if "set" in config_data["security_ttl"]:
            command += " ttl"
        elif config_data["security_ttl"].get("hops"):
            command += " ttl hops {0}".format(
                config_data["security_ttl"].get("hops"),
            )
        return command


def _tmplt_ospf_log_adjacency(config_data):
    if "log_adjacency_changes" in config_data:
        command = "log adjacency"
        if "set" in config_data["log_adjacency_changes"]:
            command += " changes"
        elif config_data["log_adjacency_changes"].get("disable"):
            command += " disable"
        elif config_data["log_adjacency_changes"].get("details"):
            command += " details"
        return command


def _tmplt_ospf_log_max_lsa(config_data):
    if "max_lsa" in config_data:
        command = "max-lsa"
        if "threshold" in config_data["max_lsa"]:
            command += " {0}".format(config_data["max_lsa"].get("threshold"))
        if "warning_only" in config_data["max_lsa"]:
            command += " warning-only {0}".format(
                config_data["max_lsa"].get("warning_only"),
            )
        if "ignore_time" in config_data["max_lsa"]:
            command += " ignore-time {0}".format(
                config_data["max_lsa"].get("ignore_time"),
            )
        if "ignore_count" in config_data["max_lsa"]:
            command += " ignore-count {0}".format(
                config_data["max_lsa"].get("ignore_count"),
            )
        if "reset_time" in config_data["max_lsa"]:
            command += " reset-time {0}".format(
                config_data["max_lsa"].get("reset_time"),
            )
        return command


def _build_on_startup_cmd(router_lsa):
    """Build the on-startup command portion."""
    on_startup = router_lsa.get("on_startup", {})

    if "wait_period" in on_startup:
        return " on-startup {wait_period}".format(**on_startup)
    elif "wait_for_bgp" in on_startup:
        return " on-startup wait-for-bgp"

    return ""


def _build_lsa_cmd(lsa_type, lsa_data):
    """Build external-lsa or summary-lsa command portion."""
    max_metric_value = lsa_data.get("max_metric_value")

    if max_metric_value:
        return " {0} {{max_metric_value}}".format(lsa_type).format(**lsa_data)

    return " {0}".format(lsa_type)


def _tmplt_ospf_max_metric(config_data):
    if "max_metric" not in config_data:
        return None

    command = "max-metric"
    max_metric = config_data["max_metric"]

    if "router_lsa" not in max_metric:
        return command

    command += " router-lsa"
    router_lsa = max_metric["router_lsa"]

    command += _build_on_startup_cmd(router_lsa)

    if "external_lsa" in router_lsa:
        command += _build_lsa_cmd("external-lsa", router_lsa["external_lsa"])

    if "include_stub" in router_lsa:
        command += " include-stub"

    if "summary_lsa" in router_lsa:
        command += _build_lsa_cmd("summary-lsa", router_lsa["summary_lsa"])

    return command


def _tmplt_ospf_distance_admin(config_data):
    if "admin_distance" in config_data:
        command = "distance"
        if config_data["admin_distance"].get("value"):
            command += " {0}".format(
                config_data["admin_distance"].get("value"),
            )
        if config_data["admin_distance"].get("source"):
            command += " {0}".format(
                config_data["admin_distance"].get("source"),
            )
        if config_data["admin_distance"].get("wildcard"):
            command += " {0}".format(
                config_data["admin_distance"].get("wildcard"),
            )
        if config_data["admin_distance"].get("access_list"):
            command += " {0}".format(
                config_data["admin_distance"].get("access_list"),
            )
        return command


def _tmplt_ospf_distance_ospf(config_data):
    if "ospf_distance" in config_data:
        command = "distance ospf"
        if config_data["ospf_distance"].get("external"):
            command += " external {0}".format(
                config_data["ospf_distance"].get("external"),
            )
        if config_data["ospf_distance"].get("inter_area"):
            command += " inter-area {0}".format(
                config_data["ospf_distance"].get("inter_area"),
            )
        if config_data["ospf_distance"].get("intra_area"):
            command += " intra-area {0}".format(
                config_data["ospf_distance"].get("intra_area"),
            )
        return command


def _tmplt_ospf_nsr(config_data):
    if "nsr" in config_data:
        command = "nsr"
        if "set" in config_data["nsr"]:
            command += " nsr"
        elif config_data["nsr"].get("disable"):
            command += " nsr {0}".format("disable")
        return command


def _tmplt_ospf_protocol(config_data):
    if "protocol_shutdown" in config_data:
        command = "protocol"
        if "set" in config_data["protocol_shutdown"]:
            command += " shutdown"
        elif config_data["shutdown"].get("host_mode"):
            command += " shutdown host-mode"
        elif config_data["shutdown"].get("on_reload"):
            command += " shutdown on-reload"
        return command


def _tmplt_microloop_avoidance(config_data):
    if "microloop_avoidance" in config_data:
        command = "microloop avoidance"
        if "protected" in config_data["microloop_avoidance"]:
            command += " protected"
        if "segment_routing" in config_data["microloop_avoidance"]:
            command += " segment_routing"
        if "rib_update_delay" in config_data["microloop_avoidance"]:
            command += " rin-update-delay {0}".config_data["microloop_avoidance"].get(
                "rib_update_delay",
            )
        return command


def _tmplt_ospf_bfd_fast_detect(config_data):
    if "bfd" in config_data:
        command = "bfd"
        if "fast_detect" in config_data["bfd"]:
            fast_detect = config_data["bfd"].get("fast_detect")
            command += " fast-detect"
            if "strict_mode" in fast_detect:
                command += " strict-mode"
        return command


def _tmplt_ospf_mpls_traffic_eng(config_data):
    if "traffic_eng" in config_data:
        command = "mpls traffic-eng"
        if "igp_intact" in config_data["traffic_eng"]:
            command += " igp-intact"
        if "ldp_sync_update" in config_data["traffic_eng"]:
            command += " ldp_sync_update"
        if "multicast_intact" in config_data["traffic_eng"]:
            command += " multicast_intact"
        if "auto_route_exclude" in config_data["traffic_eng"]:
            policy = config_data["traffic_eng"].get("autoroute_exclude")
            command += " autoroute-exlude route-policy {0}".format(
                policy.get("route_policy"),
            )
        return command


def _tmplt_ospf_authentication_md(config_data):
    command = []
    if "authentication" in config_data:
        if config_data["authentication"].get("message_digest"):
            command = "authentication message-digest"
            md = config_data["authentication"].get("message_digest")
            if md.get("keychain"):
                command += " keychain " + md.get("keychain")
        return command


def _tmplt_ospf_authentication(config_data):
    command = []
    if "authentication" in config_data:
        if config_data["authentication"].get("keychain"):
            command = "authentication keychain " + config_data["authentication"].get("keychain")
        elif config_data["authentication"].get("no_auth"):
            command = "authentication null"
        return command


def _tmplt_ospf_adjacency_stagger(config_data):
    if "adjacency_stagger" in config_data:
        command = "adjacency stagger".format(**config_data)
        if config_data["adjacency_stagger"].get("min_adjacency") and config_data[
            "adjacency_stagger"
        ].get("min_adjacency"):
            command += " {0} {1}".format(
                config_data["adjacency_stagger"].get("min_adjacency"),
                config_data["adjacency_stagger"].get("max_adjacency"),
            )
        elif config_data["adjacency_stagger"].get("disable"):
            command += " disable"
        return command


def _tmplt_ospf_adjacency_distribute_bgp_state(config_data):
    if "distribute_link_list" in config_data:
        command = "distribute link-state"
        if config_data["distribute_link_list"].get("instance_id"):
            command += "  instance-id {0}".format(
                config_data["distribute_link_list"].get("instance_id"),
            )
        elif config_data["distribute_link_list"].get("throttle"):
            command += "  throttle {0}".format(
                config_data["distribute_link_list"].get("throttle"),
            )
        return command
    elif "distribute_bgp_ls" in config_data:
        command = "distribute bgp-ls"
        if config_data["distribute_bgp_ls"].get("instance_id"):
            command += "  instance-id {0}".format(
                config_data["distribute_bgp_ls"].get("instance_id"),
            )
        elif config_data["distribute_bgp_ls"].get("throttle"):
            command += "  throttle {0}".format(
                config_data["distribute_bgp_ls"].get("throttle"),
            )
        return command


def _tmplt_ospf_capability_opaque(config_data):
    if "capability" in config_data:
        if "opaque" in config_data["capability"]:
            command = "capability opaque"
            opaque = config_data["capability"].get("opaque")
            if "disable" in opaque:
                command += "capability opaque disable"
        return command


def _tmplt_ospf_authentication_key(config_data):
    if "authentication_key" in config_data:
        command = "authentication-key".format(**config_data)
        if config_data["authentication_key"].get("password"):
            command += " {0}".format(
                config_data["authentication_key"].get("password"),
            )
        return command


def _tmplt_ospf_area_authentication(config_data):
    if "authentication" in config_data:
        command = "area {area_id} authentication".format(**config_data)
        if config_data["authentication"].get("keychain"):
            command += " keychain " + config_data["authentication"].get(
                "keychain",
            )
        elif config_data["authentication"].get("no_auth"):
            command += " null"
        return command


def _tmplt_ospf_area_authentication_md(config_data):
    if "authentication" in config_data:
        command = "area {area_id} authentication".format(**config_data)
        if "message_digest" in config_data["authentication"]:
            command = "authentication message-digest"
            md = config_data["authentication"].get("message_digest")
            if md.get("keychain"):
                command += " keychain " + md.get("keychain")
        return command


def _tmplt_ospf_area_authentication_key(config_data):
    if "authentication_key" in config_data:
        command = "area {area_id} authentication-key".format(**config_data)
        if config_data["authentication_key"].get("password"):
            command += " {0}".format(
                config_data["authentication_key"].get("password"),
            )
        return command


def _tmplt_ospf_area_mpls_ldp(config_data):
    commands = []
    if "mpls" in config_data:
        command = "area {area_id} mpls".format(**config_data)
        if config_data["mpls"].get("ldp"):
            ldp = config_data["mpls"].get("ldp")
            if "auto_config" in ldp:
                command += " auto-config"
                commands.append(command)
            if "sync" in ldp:
                command += " sync"
                commands.append(command)
            if "sync_igp_shortcuts" in ldp:
                command += " sync-igp-shortcuts"
                commands.append(command)
        return commands


def _tmplt_ospf_area_bfd(config_data):
    if "bfd" in config_data:
        command = "area {area_id} bfd".format(**config_data)
        if "minimum_interval" in config_data["bfd"]:
            command += " minimum-interval {minimum_interval}".format(**config_data["bfd"])

        if "multiplier" in config_data["bfd"]:
            command += " multiplier {multiplier}".format(**config_data["bfd"])

        return command


def _tmplt_ospf_area_bfd_fast_detect(config_data):
    if "bfd" in config_data:
        command = "area {area_id} bfd".format(**config_data)
        if "fast_detect" in config_data["bfd"]:
            fast_detect = config_data["bfd"].get("fast_detect")
            command += " fast-detect"
            if "strict_mode" in fast_detect:
                command += " strict-mode"
        return command


def _tmplt_ospf_mpls_ldp(config_data):
    commands = []
    if "mpls" in config_data:
        command = "mpls".format(**config_data)
        if config_data["mpls"].get("ldp"):
            ldp = config_data["mpls"].get("ldp")
            if "auto_config" in ldp:
                command += " auto-config"
                commands.append(command)
            if "sync" in ldp:
                command += " sync"
                commands.append(command)
            if "sync_igp_shortcuts" in ldp:
                command += " sync-igp-shortcuts"
                commands.append(command)
        return commands


def _tmplt_ospf_area_nssa(config_data):
    if "nssa" in config_data:
        command = "area {area_id} nssa".format(**config_data)
        if config_data["nssa"].get("no_redistribution"):
            command += " no-redistribution"
        if config_data["nssa"].get("no_summary"):
            command += " no-summary"
        return command


def _tmplt_ospf_area_nssa_def_info_origin(config_data):
    if "nssa" in config_data:
        command = "area {area_id} nssa".format(**config_data)
        if "default_information_originate" in config_data["nssa"]:
            command += " default-information-originate"
            def_info_origin = config_data["nssa"].get(
                "default_information_originate",
            )
            if "metric" in def_info_origin:
                command += " metric {metric}".format(
                    **config_data["nssa"]["default_information_originate"],
                )
            if "metric_type" in def_info_origin:
                command += " metric-type {metric_type}".format(
                    **config_data["nssa"]["default_information_originate"],
                )
        return command


def _tmplt_ospf_area_nssa_translate(config_data):
    if "nssa" in config_data:
        command = "area {area_id} nssa".format(**config_data)
        if config_data["nssa"].get("translate"):
            command += " translate"
            translate = config_data["nssa"].get("translate")
            if "type7" in translate:
                command += " type7"
            if translate["type7"].get("always"):
                command += " always"
        return command


def _tmplt_ospf_area_vlink_authentication(config_data):
    if "authentication" in config_data:
        command = "area {area_id} virtual-link {id} authentication".format(**config_data)
        if config_data["authentication"].get("keychain"):
            command += " keychain " + config_data["authentication"].get(
                "keychain",
            )
        elif config_data["authentication"].get("no_auth"):
            command += " null"
        return command


def _tmplt_ospf_area_vlink_authentication_md(config_data):
    if "authentication" in config_data:
        command = "area {area_id} virtual-link {id} authentication".format(**config_data)
        if config_data["authentication"].get("message_digest"):
            command = "authentication message-digest"
            md = config_data["authentication"].get("message_digest")
            if md.get("keychain"):
                command += " keychain " + md.get("keychain")
        return command


def _tmplt_ospf_area_vlink_authentication_key(config_data):
    if "authentication_key" in config_data:
        command = "area {area_id} virtual-link {id} authentication-key".format(**config_data)
        if config_data["authentication_key"].get("password"):
            command += " {0}".format(
                config_data["authentication_key"].get("password"),
            )
        return command


def _tmplt_ospf_area_stub(config_data):
    if "stub" in config_data:
        command = "area {area_id} stub".format(**config_data)
        if config_data["stub"].get("no_summary"):
            command += " no-summary"
        return command


def _tmplt_ospf_area_ranges(config_data):
    if "ranges" in config_data:
        commands = []
        for k, v in config_data["ranges"].items():
            cmd = "area {area_id} range".format(**config_data)
            temp_cmd = " {address}".format(**v)
            if "advertise" in v:
                temp_cmd += " advertise"
            elif "not_advertise" in v:
                temp_cmd += " not-advertise"
            cmd += temp_cmd
            commands.append(cmd)
        return commands


def _tmplt_prefix_suppression(config_data):
    if "prefix_suppression" in config_data:
        if "set" in config_data["prefix_suppression"]:
            command = "prefix-suppression"
        if "secondary_address" in config_data["prefix_suppression"]:
            command = "prefix-suppression secondary-address"
        return command


def _tmplt_protocol_shutdown(config_data):
    if "protocol_shutdown" in config_data:
        if "set" in config_data["protocol_shutdown"]:
            command = "protocol-shutdown"
        if "host_mode" in config_data["protocol_shutdown"]:
            command = "protocol-shutdown host-mode"
        if "on_reload" in config_data["protocol_shutdown"]:
            command = "protocol-shutdown on-reload"
        return command


def _tmplt_timers_lsa(config_data):
    if "timers" in config_data:
        command = "timers lsa"
        if "group_pacing" in config_data["timers"]["lsa"]:
            command += " group-pacing {group_pacing}".format(**config_data["timers"]["lsa"])
        if "min_arrival" in config_data["timers"]["lsa"]:
            command += " min-arrival {min_arrival}".format(**config_data["timers"]["lsa"])
        if "refresh" in config_data["timers"]["lsa"]:
            command += " refresh {refresh}".format(**config_data["timers"]["lsa"])
        return command


def _tmplt_timers_graceful_shutdown(config_data):
    if "timers" in config_data:
        command = "timers graceful-shutdown"
        if "initial_delay" in config_data["timers"]["graceful-shutdown"]:
            command += " initial delay {initial_delay}".format(
                **config_data["timers"]["graceful-shutdown"],
            )
        if "retain_routes" in config_data["timers"]["graceful-shutdown"]:
            command += " retain routes {retain_routes}".format(
                **config_data["timers"]["graceful-shutdown"],
            )
        return command


class Ospfv2Template(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(Ospfv2Template, self).__init__(
            lines=lines,
            tmplt=self,
            module=module,
        )

    # fmt: off
    PARSERS = [
        {
            "name": "pid",
            "getval": re.compile(
                r"""
                        ^router
                        \sospf\s(?P<pid>\S+)
                        $""",
                re.VERBOSE,
            ),
            "setval": "router ospf {{ process_id }}",
            "result": {
                "processes": {"{{ pid }}": {"process_id": "{{ pid }}"}},
            },
            "shared": True,
        },
        {
            "name": "cost",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \scost(?P<cost>\s\d+)
                $""",
                re.VERBOSE,
            ),

            "setval": "cost {{ cost }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "cost": "{{ cost|int }}",
                    },
                },
            },
        },
        {
            "name": "default_metric",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \sdefault-metric(?P<default_metric>\s\d+)
                $""",
                re.VERBOSE,
            ),

            "setval": "default-metric {{ default_metric }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "default_metric": "{{ default_metric|int }}",
                    },
                },
            },
        },
        {
            "name": "packet_size",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \spacket-size(?P<packet_size>\s\d+)
                $""",
                re.VERBOSE,
            ),

            "setval": "packet-size {{ packet_size }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "packet_size": "{{ packet_size|int }}",
                    },
                },
            },
        },
        {
            "name": "dead_interval",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \sdead-interval(?P<dead_interval>\s\d+)
                $""",
                re.VERBOSE,
            ),

            "setval": "dead-interval {{ dead_interval }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "dead_interval": "{{ dead_interval|int }}",
                    },
                },
            },
        },
        {
            "name": "hello_interval",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \shello-interval(?P<hello_interval>\s\d+)
                $""",
                re.VERBOSE,
            ),

            "setval": "hello-interval {{ hello_interval }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "hello_interval": "{{ hello_interval|int }}",
                    },
                },
            },
        },
        {
            "name": "priority",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \spriority(?P<priority>\s\d+)
                $""",
                re.VERBOSE,
            ),

            "setval": "priority {{ priority }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "priority": "{{ priority|int }}",
                    },
                },
            },
        },
        {
            "name": "weight",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \sweight(?P<weight>\s\d+)
                $""",
                re.VERBOSE,
            ),

            "setval": "weight {{ weight }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "weight": "{{ weight|int }}",
                    },
                },
            },
        },
        {
            "name": "retransmit_interval",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \sretransmit-interval(?P<retransmit_interval>\s\d+)
                $""",
                re.VERBOSE,
            ),

            "setval": "retransmit-interval {{ retransmit_interval }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "retransmit_interval": "{{ retransmit_interval|int }}",
                    },
                },
            },
        },
        {
            "name": "transmit_delay",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \stransmit-delay(?P<transmit_delay>\s\d+)
                $""",
                re.VERBOSE,
            ),

            "setval": "transmit-delay {{ transmit_delay }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "transmit_delay": "{{ transmit_delay|int }}",
                    },
                },
            },
        },
        {
            "name": "passive",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \spassive\s(?P<passive>\S+)
                $""",
                re.VERBOSE,
            ),

            "setval": "passive {{ passive }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "passive": "{{ passive }}",
                    },
                },
            },
        },
        {
            "name": "process.database_filter",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \sdatabase-filter
                \sall
                \sout\s(?P<database_filter>\s\S+)
                $""",
                re.VERBOSE,
            ),

            "setval": "process.database_filter",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "database_filter": "{{ database_filter }}",
                    },
                },
            },
        },
        {
            "name": "demand_circuit",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \sdemand-circuit\s(?P<demand_circuit>\S+)
                $""",
                re.VERBOSE,
            ),

            "setval": "demand-circuit {{ demand_circuit }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "demand_circuit": "{{ demand_circuit }}",
                    },
                },
            },
        },
        {
            "name": "external_out",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \sexternal-out\s(?P<external_out>\S+)
                $""",
                re.VERBOSE,
            ),

            "setval": "external-out {{ external_out }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "external_out": "{{ external_out }}",
                    },
                },
            },
        },
        {
            "name": "router_id",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \srouter-id\s(?P<router_id>\S+)
                $""",
                re.VERBOSE,
            ),

            "setval": "router-id {{ router_id }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "router_id": "{{ router_id }}",
                    },
                },
            },
        },
        {
            "name": "summary_in",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \ssummary-in\s(?P<summary_in>\S+)
                $""",
                re.VERBOSE,
            ),

            "setval": "summary-in {{ summary_in }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "summary_in": "{{ summary_in }}",
                    },
                },
            },
        },

        {
            "name": "mtu_ignore",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \smtu-ignore\s(?P<mtu_ignore>\S+)
                $""",
                re.VERBOSE,
            ),

            "setval": "mtu-ignore {{ mtu_ignore }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "mtu_ignore": "{{ mtu_ignore }}",
                    },
                },
            },
        },
        {
            "name": "flood_reduction",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \sflood-reduction\s(?P<flood_reduction>\S+)
                $""",
                re.VERBOSE,
            ),

            "setval": "flood-reduction {{ flood_reduction }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "flood_reduction": "{{ flood_reduction }}",
                    },
                },
            },
        },
        {
            "name": "loopback_stub_network",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \sloopback(?P<loopback>)
                \sstub-network\s(?P<stub_network>\S+)
                $""",
                re.VERBOSE,
            ),

            "setval": "loopback stub-network {{ stub_network }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "loopback_stub_network": "{{ loopback_stub_network }}",
                    },
                },
            },
        },
        {
            "name": "address_family_unicast",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \saddress-family(?P<address_family>)
                \sipv4(?P<ipv4>)
                \sunicast(?P<unicast>)
                $""",
                re.VERBOSE,
            ),

            "setval": "address_family_unicast",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "address_family_unicast": "{{ True if unicast is defined }}",
                    },
                },
            },
        },
        {
            "name": "default_weight",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sapply-weight(?P<apply_weight>)
                    \sdefault-weight(?P<default_weight>\s\d+)
                    $""",
                re.VERBOSE,
            ),
            "setval": "apply-weight default-weight {{ default_weight }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "apply_weight": {
                            "default_weight": "{{ default_weight|int }}",
                        },
                    },
                },
            },
        },
        {
            "name": "bandwidth",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sapply-weight(?P<apply_weight>)
                    \sbandwidth(?P<bandwidth>\s\d+)?
                    $""",
                re.VERBOSE,
            ),
            "setval": "apply-weight bandwidth {{ bandwidth }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "apply_weight": {
                            "bandwidth": "{{ bandwidth|int }}",
                        },
                    },
                },
            },
        },
        {
            "name": "adjacency_stagger",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sadjacency(?P<adjacency>)
                    \sstagger(?P<stagger>)
                    (\s(?P<min_adjacency>\d+))?
                    (\s(?P<max_adjacency>\d+))?
                    (\sdisable(?P<disable>\S+))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_adjacency_stagger,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "adjacency_stagger": {
                            "min_adjacency": "{{ min_adjacency|int }}",
                            "max_adjacency": "{{ max_adjacency }}",
                            "disable": "{{ True if disable is defined }}",
                        },
                    },
                },
            },
        },
        {
            "name": "authentication",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sauthentication(?P<auth>)
                    (\skeychain\s(?P<keychain>\S+)*)?
                    (\snull(?P<no_auth>))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_authentication,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "authentication": {
                            "no_auth": "{{ True if no_auth is defined }}",
                            "keychain": "{{ keychain }}",
                        },
                    },
                },
            },
        },
        {
            "name": "authentication.message_digest",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sauthentication(?P<auth>)
                    \smessage-digest(?P<md>)
                    \skeychain\s(?P<md_key>\S+)
                    *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_authentication_md,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "authentication": {
                            "message_digest": {
                                "keychain": "{{ md_key }}",
                                "set": "{{ True if md is defined and md_key is undefined }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "default_information_originate",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sdefault-information(?P<default_information>)
                    (\soriginate(?P<originate>))?
                    (\salways(?P<always>))?
                    (\smetric\s(?P<metric>\d+))?
                    (\smetric-type\s(?P<metric_type>\d+))?
                    (\sroute_policy\s(?P<route_policy>)\S+)?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_default_information,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "default_information_originate": {
                            "always": "{{ True if always is defined }}",
                            "metric": "{{ metric|int }}",
                            "metric_type": "{{ metric_type|int }}",
                            "route_policy": "{{ route_policy }}",
                            "set": "{{ True if default_information is defined and always is undefined and metric "
                                   "is undefined and metric_type is undefined and route_policy is undefined }}",
                        },
                    },
                },
            },
        },
        {
            "name": "auto_cost",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sauto-cost(?P<auto_cost>)
                    (\sreference-bandwidth\s(?P<reference_bandwidth>\d+))?
                    (\sdisable(?P<disable>))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_auto_cost,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "auto_cost": {
                            "disable": "{{ True if disable is defined }}",
                            "reference_bandwidth": "{{ reference_bandwidth|int }}",
                        },
                    },
                },
            },
        },
        {
            "name": "bfd",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sbfd(?P<bfd>)
                    (\sminimum-interval\s(?P<minimum_interval>\d+))?
                    (\smultiplier\s(?P<multiplier>\d+))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_bfd,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "bfd": {
                            "minimum_interval": "{{ minimum_interval|int }}",
                            "multiplier": "{{ multiplier|int }}",
                        },
                    },
                },
            },
        },
        {
            "name": "bfd.fast_detect",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sbfd(?P<bfd>)
                    \sfast-detect(?P<fast_detect>)
                    (\s(?P<disable>disable))?
                    (\s(?P<strict_mode>strict-mode))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_bfd_fast_detect,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "bfd": {
                            "fast_detect": {
                                "set": "{{ True if disable is undefined and strict_mode is undefined }}",
                                "strict_mode": "{{ True if strict_mode is defined }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "security",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \ssecurity(?P<security>)
                    \sttl(?P<ttl>)?
                    (\shops\s(?P<hops>\d+))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_security,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "security_ttl": {
                            "set": "{{ True if ttl is defined and hops is undefined }}",
                            "hops": "{{ hops }}",
                        },
                    },
                },
            },
        },
        {
            "name": "nsr",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \snsr(?P<nsr>)
                    \sdisable(?P<disable>)?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_nsr,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "nsr": {
                            "set": "{{ True if nsr is defined and disable is undefined }}",
                            "disable": "{{ True if disable is defined }}",
                        },
                    },
                },
            },
        },
        {
            "name": "protocol",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sprotocol(?P<protocol>)
                    \s(shutdown(?P<shutdown>))
                    (\shost-mode(?P<host_mode>))?
                    (\son-reload\s(?P<on_reload>\d+))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_protocol,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "protocol_shutdown": {
                            "set": "{{ True if shutdown is defined and host-mode is undefined and on_reload is undefined  }}",
                            "host_mode": "{{ True if host_mode is defined }}",
                            "on_reload": "{{ True if on_reload is defined }}",
                        },
                    },
                },
            },
        },
        {
            "name": "capability",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \scapability(?P<capability>)
                    (\stype7\s(?P<type7>\S+))?
                    $""",
                re.VERBOSE,
            ),
            "setval": "capability type7 {{ type7 }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "capability": {
                            "type7": "{{ type7 }}",
                        },
                    },
                },
            },
        },
        {
            "name": "capability.opaque",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \scapability(?P<capability>)?
                    \sopaque(?P<opaque>)
                    (\sdisable(?P<disable>))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_capability_opaque,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "capability": {
                            "opaque": {
                                "disable": "{{ True if disable is defined }}",
                                "set": "{{ True if opaque is defined and disable is undefined }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "admin_distance",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sdistance\s(?P<value>d+)
                    \s(?P<source>\S+)
                    \s(?P<wildcard>\S+)
                    (\s(?P<access_list>\S+))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_distance_admin,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "distance": {
                            "admin_distance": {
                                "value": "{{ value|int }}",
                                "source": "{{ source }}",
                                "wildcard": "{{ wildcard }}",
                                "access_list": "{{ access_list }}",
                            },
                        },
                    },
                },
            },
        },

        {
            "name": "ospf_distance",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sdistance(?P<value>)
                    \sospf(?P<ospf>)
                    (\sexternal\s(?P<external>\d+))?
                    (\sinter-area\s(?P<inter_area>\d+))?
                    (\sintra-area\s(?P<intra_area>\d+))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_distance_ospf,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "distance": {
                            "ospf_distance": {
                                "external": "{{ external|int }}",
                                "inter_area": "{{ inter_area|int }}",
                                "intra_area": "{{ intra_area|int }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "authentication_key",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sauthentication-key(?P<auth_key>)
                    (\s(?P<password>\S+))?
                    (\sclear\s(?P<clear>)\S+)?
                    (\sencrypted(?P<encrypted>\S+))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_authentication_key,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "authentication_key": {
                            "clear": "{{ clear }}",
                            "encrypted": "{{ encrypted}}",
                            "password": "{{ password if clear is undefined and encrypted is undefined }}",
                        },
                    },
                },
            },
        },
        {
            "name": "area.default_cost",
            "getval": re.compile(
                r"""
                   ^router
                   \sospf\s(?P<pid>\S+)
                   \sarea\s(?P<area_id>\S+)
                   \sdefault-cost\s(?P<default_cost>\d+)
                   $""",
                re.VERBOSE,
            ),

            "setval": "area {{ area_id }} default-cost {{ default_cost }}",
            "compval": "default_cost",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "default_cost": "{{ default_cost|int }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.dead_interval",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \sarea\s(?P<area_id>\S+)
                \sdead-interval\s(?P<dead_interval>\d+)
                $""",
                re.VERBOSE,
            ),

            "setval": "area {{ area_id }} dead-interval {{ dead_interval }}",
            "compval": "dead_interval",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "dead_interval": "{{ dead_interval|int }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.hello_interval",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \sarea\s(?P<area_id>\S+)
                \shello-interval\s(?P<hello_interval>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": "area {{ area_id }} hello-interval {{ hello_interval }}",
            "compval": "hello_interval",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "hello_interval": "{{ hello_interval|int }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.transmit_delay",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \sarea\s(?P<area_id>\S+)
                \stransmit-delay\s(?P<transmit_delay>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": "area {{ area_id }} transmit-delay {{ transmit_delay }}",
            "compval": "transmit_delay",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "transmit_delay": "{{ transmit_delay|int }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.cost",
            "getval": re.compile(
                r"""
                   ^router
                   \sospf\s(?P<pid>\S+)
                   \sarea\s(?P<area_id>\S+)
                   \scost\s(?P<cost>\d+)
                   $""",
                re.VERBOSE,
            ),
            "setval": "area {{ area_id }} cost {{ cost }}",
            "compval": "cost",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "cost": "{{ cost|int }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.priority",
            "getval": re.compile(
                r"""
                   ^router
                   \sospf\s(?P<pid>\S+)
                   \sarea\s(?P<area_id>\S+)
                   \spriority\s(?P<priority>\d+)
                   $""",
                re.VERBOSE,
            ),
            "setval": "area {{ area_id }} priority {{ priority }}",
            "compval": "priority",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "priority": "{{ priority|int }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.weight",
            "getval": re.compile(
                r"""
                   ^router
                   \sospf\s(?P<pid>\S+)
                   \sarea\s(?P<area_id>\S+)
                   \sweight\s(?P<weight>\d+)
                   $""",
                re.VERBOSE,
            ),
            "setval": "area {{ area_id }} weight {{ weight }}",
            "compval": "weight",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "weight": "{{ weight|int }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.packet_size",
            "getval": re.compile(
                r"""
                   ^router
                   \sospf\s(?P<pid>\S+)
                   \sarea\s(?P<area_id>\S+)
                   \spacket-size\s(?P<packet_size>\d+)
                   $""",
                re.VERBOSE,
            ),
            "setval": "area {{ area_id }} packet-size {{ packet_size }}",
            "compval": "packet_size",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "packet_size": "{{ packet_size|int }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.summary_in",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \sarea\s(?P<area_id>\S+)
                \ssummary-in\s(?P<summary_in>\S+)
                $""",
                re.VERBOSE,
            ),

            "setval": "area {{ area_id }} summary-in {{ summary_in }}",
            "compval": "summary_in",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "summary_in": "{{ summary_in }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.demand_circuit",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \sarea\s(?P<area_id>\S+)
                \sdemand-circuit\s(?P<demand_circuit>\S+)
                $""",
                re.VERBOSE,
            ),

            "setval": "area {{ area_id }} demand-circuit {{ demand_circuit }}",
            "compval": "demand_circuit",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "demand_circuit": "{{ demand_circuit }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.passive",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \sarea\s(?P<area_id>\S+)
                \spassive\s(?P<passive>\S+)
                $""",
                re.VERBOSE,
            ),

            "setval": "area {{ area_id }} passive {{ passive }}",
            "compval": "passive",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "passive": "{{ passive }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.external_out",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \sarea\s(?P<area_id>\S+)
                \sexternal-out\s(?P<external_out>\S+)
                $""",
                re.VERBOSE,
            ),

            "setval": "area {{ area_id }} external-out {{ external_out }}",
            "compval": "passive",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "external_out": "{{ external_out }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.mtu_ignore",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \sarea\s(?P<area_id>\S+)
                \smtu-ignore\s(?P<mtu_ignore>\S+)
                $""",
                re.VERBOSE,
            ),

            "setval": "area {{ area_id }} mtu-ignore {{ mtu_ignore }}",
            "compval": "mtu_ignore",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "mtu_ignore": "{{ mtu_ignore }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.authentication",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sarea\s(?P<area_id>\S+)
                    \sauthentication(?P<auth>)
                    (\skeychain\s(?P<keychain>\S+))?
                    (\snull(?P<no_auth>))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_area_authentication,
            "compval": "authentication",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "authentication": {
                                    "no_auth": "{{ True if no_auth is defined }}",
                                    "keychain": "{{ keychain }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.authentication_key",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sarea\s(?P<area_id>\S+)
                    \sauthentication-key(?P<auth_key>)
                    (\s(?P<password>\S+))?
                    (\sclear\s(?P<clear>)\S+)?
                    (\sencrypted(?P<encrypted>\S+))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_area_authentication_key,
            "compval": "authentication_key",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "authentication_key": {
                                    "clear": "{{ clear }}",
                                    "encrypted": "{{ encrypted}}",
                                    "password": "{{ password if clear is undefined and encrypted is undefined }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.authentication.message_digest",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sarea\s(?P<area_id>\S+)
                    \sauthentication(?P<auth>)
                    \smessage-digest(?P<md>)
                    \skeychain(?P<md_key>\s\S+)
                    *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_area_authentication_md,
            "compval": "authentication.message_digest",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "authentication": {
                                    "message_digest": {
                                        "keychain": "{{ md_key }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.mpls_traffic_eng",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sarea\s(?P<area_id>\S+)
                    \smpls(?P<mpls>)
                    \straffic-end(?P<traffic_eng>)
                    $""",
                re.VERBOSE,
            ),
            "setval": "area {{ area_id }} mpls traffic-eng",
            "compval": "mpls_traffic_eng",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "mpls": {
                                    "traffic_eng": "{{ True if traffic_eng is defined }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.mpls_ldp",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sarea\s(?P<area_id>\S+)
                    \smpls(?P<mpls>)
                    (\sauto-config(?P<auto_config>))?
                    (\ssync(?P<sync>))?
                    (\ssync-igp-shortcuts(?P<syn_igp_shortcuts>))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_area_mpls_ldp,
            "compval": "mpls_ldp",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "mpls": {
                                    "ldp": {
                                        "auto_config": "{{ True if auto_config is defined }}",
                                        "sync": "{{ True if sync is defined }}",
                                        "sync_igp_shortcuts": "{{ True if sync_igp_shortcuts is defined }}",

                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.bfd",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sarea\s(?P<area_id>\S+)
                    \sbfd(?P<bfd>)
                    (\sminimum-interval\s(?P<minimum_interval>\d+))?
                    (\smultiplier\s(?P<multiplier>\d+))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_area_bfd,
            "compval": "bfd",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "bfd": {
                                    "minimum_interval": "{{ minimum_interval|int }}",
                                    "multiplier": "{{ multiplier|int }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.bfd.fast_detect",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sbfd(?P<bfd>)
                    \sarea(?P<area_id>)
                    \sfast-detect(?P<fast_detect>)
                    (\s(?P<disable>disable))?
                    (\s(?P<strict_mode>strict-mode))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_area_bfd_fast_detect,
            "compval": "bfd.fast_detect",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "bfd": {
                                    "fast_detect": {
                                        "set": "{{ True if disable is undefined and strict_mode is undefined }}",
                                        "strict_mode": "{{ True if strict_mode is defined }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.stub",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \sarea\s(?P<area_id>\S+)
                \sstub(?P<nssa>)
                (\sno-summary(?P<no_sum>))?
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_area_stub,
            "compval": "stub",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "area_id": "{{ area_id }}",
                            "stub": {
                                "set": "{{ True if stub is defined and no_summary is undefined }}",
                                "no_summary": "{{ True if no_summary is defined }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.nssa",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \sarea\s(?P<area_id>\S+)
                \snssa(?P<nssa>)
                (\sno-redistribution(?P<no_redis>))?
                (\sno-summary(?P<no_sum>))?
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_area_nssa,
            "compval": "nssa",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "area_id": "{{ area_id }}",
                            "nssa": {
                                "set": "{{ True if nssa is defined and no_summary is undefined and no_redis is undefined }}",
                                "no_summary": "{{ True if no_summary is defined }}",
                                "no_redistribution": "{{ True if no_redis is defined }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.nssa.default_information_originate",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \sarea\s(?P<area_id>\S+)
                \snssa(?P<nssa>)
                (\sno-redistribution(?P<no_redis>))?
                (\sdefault-information-originate(?P<def_info_origin>))?
                (\smetric\s(?P<metric>\d+))?
                (\smetric-type\s(?P<metric_type>\d+))?
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_area_nssa_def_info_origin,
            "compval": "nssa.default_information_originate",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "nssa": {
                                    "default_information_originate": {
                                        "metric": "{{ metric|int }}",
                                        "metric_type": "{{ metric_type|int }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.ranges",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sarea\s(?P<area_id>\S+)
                    \srange(?P<range>)
                    \s(?P<address>\S+)
                    (\sadvertise(?P<advertise>))
                    (\snot-advertise(?P<not_advertise>))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_area_ranges,
            "compval": "ranges",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "ranges": [
                                    {
                                        "address": "{{ address }}",
                                        "advertise": "{{ True if advertise is defined }}",
                                        "not_advertise": "{{ True if not_advertise is defined }}",
                                    },
                                ],
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.nssa.translate",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \sarea\s(?P<area_id>\S+)
                \snssa(?P<nssa>)
                \stranslate(?P<translate>)
                \stype7(?P<type7>)
                \salways\s(?P<always>)
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_area_nssa_translate,
            "compval": "nssa.translate",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "area_id": "{{ area_id }}",
                            "nssa": {
                                "translate": {
                                    "type7": {
                                        "always": "{{ True if always is defined }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "virtual_link.hello_interval",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sarea\s(?P<area_id>\S+)
                    \svirtual-link\s(?P<id>\S+)
                    \shello-interval\s(?P<hello_interval>\d+)
                    $""",
                re.VERBOSE,
            ),
            "setval": "area {{ area_id }} virtual-link {{ id }} hello-interval {{ hello_interval }}",
            "compval": "hello_interval",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "virtual_link": {
                                    "{{ id }}":
                                        {
                                            "id": "{{ id }}",
                                            "hello_interval": "{{ hello_interval|int }}",
                                        },

                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "virtual_link.dead_interval",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sarea\s(?P<area_id>\S+)
                    \svirtual-link\s(?P<id>\S+)
                    \sdead-interval\s(?P<dead_interval>\d+)
                    $""",
                re.VERBOSE,
            ),
            "setval": "area {{ area_id }} virtual-link {{ id }} dead-interval {{ dead_interval }}",
            "compval": "dead_interval",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "virtual_link": {
                                    "{{ id }}": {
                                        "id": "{{ id }}",
                                        "dead_interval": "{{ dead_interval|int }}",
                                    },

                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "virtual_link.retransmit_interval",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sarea\s(?P<area_id>\S+)
                    \svirtual-link\s(?P<id>\S+)
                    \sretransmit-interval\s(?P<retransmit_interval>\d+)
                    $""",
                re.VERBOSE,
            ),
            "setval": "area {{ area_id }} virtual-link {{ id }} retransmit-interval {{ retransmit_interval }}",
            "compval": "retransmit_interval",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "virtual_link": {
                                    "{{ id }}": {
                                        "id": "{{ id }}",
                                        "retransmit_interval": "{{ retransmit_interval|int }}",
                                    },
                                },

                            },
                        },
                    },
                },
            },
        },
        {
            "name": "virtual_link.authentication",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sarea\s(?P<area_id>\S+)
                    \savirtual-link\s(?P<id>\S+)
                    \sauthentication(?P<auth>)
                    (\skeychain\s(?P<keychain>\S+))?
                    (\snull(?P<no_auth>))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_area_vlink_authentication,
            "compval": "authentication",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "authentication": {
                                    "no_auth": "{{ True if no_auth is defined }}",
                                    "keychain": "{{ keychain }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "virtual_link.authentication_key",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sarea\s(?P<area_id>\S+)
                    \svirtual-link\s(?P<id>\S+)
                    \sauthentication-key(?P<auth_key>)
                    (\s(?P<password>\S+))?
                    (\sclear\s(?P<clear>)\S+)?
                    (\sencrypted(?P<encrypted>\S+))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_area_vlink_authentication_key,
            "compval": "authentication_key",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "virtual_link": {
                                    "{{ id }}": {
                                        "authentication_key": {
                                            "clear": "{{ clear }}",
                                            "encrypted": "{{ encrypted}}",
                                            "password": "{{ password if clear is undefined and encrypted is undefined }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "virtual_link.authentication.message_digest",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sarea\s(?P<area_id>\S+)
                    \svirtual-link\s(?P<id>\S+)
                    \sauthentication(?P<auth>)
                    \smessage-digest(?P<md>)
                    \skeychain(?P<md_key>\s\S+)
                    *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_area_vlink_authentication_md,
            "compval": "authentication.message_digest",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "areas": {
                            "{{ area_id }}": {
                                "area_id": "{{ area_id }}",
                                "virtual_link": {
                                    "{{ id }}": {
                                        "authentication": {
                                            "message_digest": {
                                                "keychain": "{{ md_key }}",
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "link_down_fast_detect",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \slink-down
                    \sfast-detect(?P<fast_detect>)
                    $""",
                re.VERBOSE,
            ),
            "setval": "link-down fast-detect",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "link_down_fast_detect": "{{ True if fast_detect is defined }}",
                    },
                },
            },
        },
        {
            "name": "nsr",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \snsr
                    \sdisable(?P<disable>)
                    $""",
                re.VERBOSE,
            ),
            "setval": "nsr disable",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "disable": "{{ True if disable is defined }}",
                    },
                },
            },
        },
        {
            "name": "database_filter",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \sdatabase-filter
                \sall
                \sout\s(?P<outing>\S+)
                $""",
                re.VERBOSE,
            ),

            "setval": "database-filter all out {{ outing }}",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "outing": "{{ outing }}",
                    },
                },
            },
        },
        {
            "name": "distribute_link_state",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sdistribute(?P<distribute>)
                    \slink-state(?P<link_state>)
                    (\sinstance-id(?P<inst_id>\d+))?
                    (\sthrottle(?P<throttle>\d+))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_adjacency_distribute_bgp_state,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "distribute_link_list": {
                            "instance_id": "{{ inst_id|int }}",
                            "throttle": "{{ throttle }}",
                        },
                    },
                },
            },
        },
        {
            "name": "distribute_bgp_ls",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sdistribute(?P<distribute>)
                    \sbgp-ls(?P<bgp_ls>)
                    (\sinstance-id(?P<inst_id>\d+))?
                    (\sthrottle(?P<throttle>\d+))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_adjacency_distribute_bgp_state,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "distribute_bgp_ls": {
                            "instance_id": "{{ inst_id|int }}",
                            "throttle": "{{ throttle }}",
                        },
                    },
                },
            },
        },
        {
            "name": "log_adjacency_changes",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \slog(?P<security>)
                    \sadjacency(?P<adjacency>)?
                    (\schanges(?P<changes>))?
                    (\sdisable(?P<disable>))?
                    (\sdetails(?P<details>))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_log_adjacency,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "log_adjacency_changes": {
                            "set": "{{ True if changes is defined }}",
                            "disable": "{{ True if disable is defined }}",
                            "details": "{{ True if details is defined }}",
                        },
                    },
                },
            },
        },
        {
            "name": "max_lsa",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    (\smax-lsa\s(?P<threshold>\d+))?
                    (\swarning-only\s(?P<warning_only>\d+)?
                    (\signore-time\s(?P<ignore_time>\d+))?
                    (\signore-count\s(?P<ignore_count>\d+))?
                    (\sreset-time\s(?P<reset_time>)\d+))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_log_max_lsa,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "max_lsa": {
                            "threshold": "{{ threshold|int }}",
                            "warning_only": "{{ warning_only|int }}",
                            "ignore_time": "{{ ignore_time|int }}",
                            "ignore_count": "{{ ignore_count|int }}",
                            "reset_time": "{{ reset_time|int }}",
                        },
                    },
                },
            },
        },
        {
            "name": "max_metric",
            "getval": re.compile(
                r"""
                ^router
                \sospf\s(?P<pid>\S+)
                \smax-metric
                \s*(?P<router_lsa>router-lsa)
                \s*(on-startup(?:\s+(?P<wait_for_bgp>wait-for-bgp)|\s+(?P<wait_period>\d+)))?
                (?:
                |\s+(?P<external_lsa>external-lsa)(?:\s+(?P<max_metric_value>\d+))?
                |\s+(?P<include_stub>include-stub)
                |\s+(?P<summary_lsa>summary-lsa)(?:\s+(?P<sum_lsa_max_metric_value>\d+))?
                )*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_max_metric,
            "result": {
                "processes": {
                    '{{ pid  }}': {
                        "max_metric": {
                            "router_lsa": {
                                "set": "{{ True if router_lsa is defined }}",
                                "external_lsa": {
                                    "set": "{{ True if external_lsa is defined and max_metric_value is undefined else None }}",
                                    "max_metric_value": "{{ max_metric_value }}",
                                },
                                "include_stub": "{{ not not include_stub }}",
                                "on_startup": {
                                    "wait_period": "{{ wait_period }}",
                                    "wait_for_bgp": "{{ not not wait_for_bgp }}",
                                },
                                "summary_lsa": {
                                    "set": "{{ True if summary_lsa is defined and sum_lsa_max_metric_value is undefined else None }}",
                                    "max_metric_value": "{{ sum_lsa_max_metric_value }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mpls_ldp",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \smpls(?P<mpls>)
                    (\sauto-config(?P<auto_config>))?
                    (\ssync(?P<sync>))?
                    (\ssync-igp-shortcuts(?P<syn_igp_shortcuts>))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_mpls_ldp,
            "compval": "mpls_ldp",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "mpls": {
                            "ldp": {
                                "auto_config": "{{ True if auto_config is defined }}",
                                "sync": "{{ True if sync is defined }}",
                                "sync_igp_shortcuts": "{{ True if sync_igp_shortcuts is defined }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "microloop_avoidance",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \smicroloop(?P<microloop>)
                    \savoidance(?P<avoidance>)
                    (\s(?P<protected>protected))?
                    (\s(?P<segment_routing>segment-routing))?
                    (\srib-update-delay\s(?P<rib_update_delay>\d+))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_microloop_avoidance,
            "compval": "microloop_avoidance",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "microloop_avoidance": {
                            "protected": "{{ True if protected is defined }}",
                            "segment_routing": "{{ True if segment_routing is defined }}",
                            "rib_update_delay": "{{ rib_update_delay }}",
                        },
                    },
                },
            },
        },
        {
            "name": "mpls_traffic_eng",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \smpls(?P<mpls>)
                    \straffic-end(?P<traffic_eng>)
                    (\sautoroute-exclude(?P<autoroute>))?
                    (\sroute-policy(?P<route_policy>\S+))?
                    (\s(?P<igp_intact>igp_intact))?
                    (\s(?P<ldp_sync_update>ldp-sync-update))?
                    (\s(?P<multicast_intact>multicast-intact))?
                    (\srouter-id\s(?P<router_id>\S+))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_mpls_traffic_eng,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "mpls": {
                            "autoroute_exclude": {
                                "route_policy": "{{ route_policy }}",
                            },
                            "igp_intact": "{{ True if igp_intact is defined }}",
                            "ldp_sync_update": "{{ True if ldp_sync_update is defined }}",
                            "multicast_intact": "{{ True if multicast_intact is defined }}",
                            "router_id": "{{ router_id }}",
                        },
                    },
                },
            },
        },
        {
            "name": "prefix_suppression",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sprefix-suppression(?P<prefix_suppression>)
                    (\s(?P<secondary_address>secondary-address))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_prefix_suppression,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "prefix_suppression": {
                            "set": "{{ True if prefix_suppression is defined and secondary_address is undefined }}",
                            "secondary_address": "{{ True if secondary_address is defined }}",
                        },
                    },
                },
            },
        },
        {
            "name": "protocol_shutdown",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \sprotocol-shutdown(?P<protocol_shutdown>)
                    (\s(?P<host_mode>host-mode))?
                    (\s(?P<on_reload>on-reload))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_protocol_shutdown,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "protocol_shutdown": {
                            "set": "{{ True if protocol_shutdown is defined and host_mode is undefined and on_reload is undefined }}",
                            "host_mode": "{{ True if host_mode is defined }}",
                            "on_reload": "{{ True if on_reload is defined }}",
                        },
                    },
                },
            },
        },
        {
            "name": "timers.lsa",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \stimers
                    \slsa
                    (\sgroup-pacing\s(?P<group_pacing>\d+))?
                    (\smin-arrival\s(?P<min_arrival>\d+))?
                    (\srefresh\s(?P<refresh>\d+))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_timers_lsa,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "timers": {
                            "lsa": {
                                "group_pacing": "{{ group_pacing|int }}",
                                "min_arrival": "{{ min_arrival|int }}",
                                "refresh": "{{ refresh|int }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "timers.graceful_shutdown",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \stimers
                    \sgraceful_shutdown
                    (\sinitial delay\s(?P<initial_delay>\d+))?
                    (\sretain routes\s(?P<retain_routes>\d+))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_timers_graceful_shutdown,
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "timers": {
                            "graceful_shutdown": {
                                "initial_delay": "{{ initial_delay|int }}",
                                "retain_routes": "{{ retain_routes|int }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "throttle.spf",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \stimers
                    \sthrottle
                    \sspf
                    (\s(?P<change_delay>\d+))
                    (\s(?P<second_delay>\d+))
                    (\s(?P<max_wait>\d+))
                    $""",
                re.VERBOSE,
            ),
            "setval": "timers throttle spf {{ throttle.spf.change_delay }}  {{ throttle.spf.second_delay }} {{ throttle.spf.max_wait }}",
            "compval": "throttle.lsa_all",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "timers": {
                            "throttle": {
                                "lsa_all": {
                                    "initial_delay": "{{ initial_delay }}",
                                    "min_delay": "{{ min_delay }}",
                                    "max_delay": "{{ max_delay }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "throttle.lsa_all",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \stimers
                    \sthrottle
                    \slsa
                    \sall
                    (\s(?P<initial_delay>\d+))
                    (\s(?P<min_delay>\d+))
                    (\s(?P<max_delay>\d+))
                    $""",
                re.VERBOSE,
            ),
            "setval": "timers throttle lsa all {{ throttle.lsa_all.initial_delay }} {{ throttle.lsa_all.min_delay }} {{ throttle.lsa_all.max_delay }}",
            "compval": "throttle.lsa_all",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "timers": {
                            "throttle": {
                                "lsa_all": {
                                    "initial_delay": "{{ initial_delay }}",
                                    "min_delay": "{{ min_delay }}",
                                    "max_delay": "{{ max_delay }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "throttle.fast_reroute",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \stimers
                    \sthrottle
                    \sfast-reroute\s(?P<fast_reroute>\d+)
                    $""",
                re.VERBOSE,
            ),
            "setval": "timers throttle fast-reroute {{ fast_reroute }}",
            "compval": "throttle.fast_reroute",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "timers": {
                            "throttle": {
                                "fast_reroute": "{{ fast_reroute }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "timers.pacing_flood",
            "getval": re.compile(
                r"""
                    ^router
                    \sospf\s(?P<pid>\S+)
                    \stimers
                    \spacing
                    \sflood\s(?P<pacing_flood>\d+)
                    $""",
                re.VERBOSE,
            ),
            "setval": "timers pacing flood {{ pacing_flood }}",
            "compval": "timers.pacing_flood",
            "result": {
                "processes": {
                    "{{ pid }}": {
                        "timers": {
                            "pacing_flood": "{{ pacing_flood }}",

                        },
                    },
                },
            },
        },
    ]
    # fmt: on



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage Check Point Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: cp_mgmt_vpn_community_meshed
short_description: Manages vpn-community-meshed objects on Check Point over Web Services API
description:
  - Manages vpn-community-meshed objects on Check Point devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
  - Available from R80 management version.
version_added: "1.0.0"
author: "Or Soffer (@chkp-orso)"
options:
  name:
    description:
      - Object name.
    type: str
    required: True
  disable_nat:
    description:
      - Indicates whether to disable NAT inside the VPN Community.
      - Available from R82 JHF management version.
    type: bool
    version_added: "6.5.0"
  encrypted_traffic:
    description:
      - Encrypted traffic settings.
      - Available from R82 JHF management version.
    type: dict
    version_added: "6.5.0"
    suboptions:
      enabled:
        description:
          - Indicates whether to accept all encrypted traffic.
        type: bool
  encryption_method:
    description:
      - The encryption method to be used.
      - Available from R80.10 management version.
    type: str
    choices: ['prefer ikev2 but support ikev1', 'ikev2 only', 'ikev1 for ipv4 and ikev2 for ipv6 only']
  encryption_suite:
    description:
      - The encryption suite to be used.
      - Available from R80.10 management version.
    type: str
    choices: ['suite-b-gcm-256', 'custom', 'vpn b', 'vpn a', 'suite-b-gcm-128']
  excluded_services:
    description:
      - Collection of services that are excluded from the community identified by the name or UID.<br> Connections with these services will not be
        encrypted and will not match rules specifying the community in the VPN community.
      - Available from R82 JHF management version.
    type: list
    elements: str
    version_added: "6.5.0"
  gateways:
    description:
      - Collection of Gateway objects identified by the name or UID.
    type: list
    elements: str
  ike_phase_1:
    description:
      - Ike Phase 1 settings. Only applicable when the encryption-suite is set to [custom].
      - Available from R80.10 management version.
    type: dict
    suboptions:
      data_integrity:
        description:
          - The hash algorithm to be used.
        type: str
        choices: ['aes-xcbc', 'sha1', 'sha256', 'sha384', 'sha512', 'md5']
      diffie_hellman_group:
        description:
          - The Diffie-Hellman group to be used.
        type: str
        choices: ['group-1', 'group-2', 'group-5', 'group-14', 'group-19', 'group-20']
      encryption_algorithm:
        description:
          - The encryption algorithm to be used.
        type: str
        choices: ['cast', 'aes-256', 'des', 'aes-128', '3des']
  ike_phase_2:
    description:
      - Ike Phase 2 settings. Only applicable when the encryption-suite is set to [custom].
      - Available from R80.10 management version.
    type: dict
    suboptions:
      data_integrity:
        description:
          - The hash algorithm to be used.
        type: str
        choices: ['aes-xcbc', 'sha1', 'sha256', 'sha384', 'sha512', 'md5']
      encryption_algorithm:
        description:
          - The encryption algorithm to be used.
        type: str
        choices: ['cast', 'aes-gcm-256', 'cast-40', 'aes-256', 'des', 'aes-128', '3des', 'des-40cp', 'aes-gcm-128', 'none']
  permanent_tunnels:
    description:
      - Permanent tunnels properties.
      - Available from R82 JHF management version.
    type: dict
    version_added: "6.5.0"
    suboptions:
      set_permanent_tunnels:
        description:
          - Indicates which tunnels to set as permanent.
        type: str
        choices: ['on all tunnels in the community', 'on all tunnels of specific gateways', 'on specific tunnels in the community', 'off']
      gateways:
        description:
          - List of gateways to set all their tunnels to permanent with specified track options. Will take effect only if set-permanent-tunnels-on
            is set to all-tunnels-of-specific-gateways.
        type: list
        elements: dict
        suboptions:
          gateway:
            description:
              - Gateway to set all is tunnels to permanent with specified track options.<br> Identified by name or UID.
            type: str
          track_options:
            description:
              - Indicates whether to use the community track options or to override track options for the permanent tunnels.
            type: str
            choices: ['according to community track options', 'override track options']
          override_tunnel_down_track:
            description:
              - Gateway tunnel down track option. Relevant only if the track-options is set to 'override track options'.
            type: str
            choices: ['none', 'log', 'popup alert', 'mail alert', 'snmp trap alert', 'user defined alert no.1', 'user defined alert no.2',
                     'user defined alert no.3']
          override_tunnel_up_track:
            description:
              - Gateway tunnel up track option. Relevant only if the track-options is set to 'override track options'.
            type: str
            choices: ['none', 'log', 'popup alert', 'mail alert', 'snmp trap alert', 'user defined alert no.1', 'user defined alert no.2',
                     'user defined alert no.3']
      tunnels:
        description:
          - List of tunnels to set as permanent with specified track options. Will take effect only if set-permanent-tunnels-on is set to
            specific-tunnels-in-the-community.
        type: list
        elements: dict
        suboptions:
          first_tunnel_endpoint:
            description:
              - First tunnel endpoint (center gateway). Identified by name or UID.
            type: str
          second_tunnel_endpoint:
            description:
              - Second tunnel endpoint (center gateway for meshed VPN community and satellite gateway for star VPN community). Identified by name or UID.
            type: str
          track_options:
            description:
              - Indicates whether to use the community track options or to override track options for the permanent tunnels.
            type: str
            choices: ['according to community track options', 'override track options']
          override_tunnel_down_track:
            description:
              - Gateway tunnel down track option. Relevant only if the track-options is set to 'override track options'.
            type: str
            choices: ['none', 'log', 'popup alert', 'mail alert', 'snmp trap alert', 'user defined alert no.1', 'user defined alert no.2',
                     'user defined alert no.3']
          override_tunnel_up_track:
            description:
              - Gateway tunnel up track option. Relevant only if the track-options is set to 'override track options'.
            type: str
            choices: ['none', 'log', 'popup alert', 'mail alert', 'snmp trap alert', 'user defined alert no.1', 'user defined alert no.2',
                     'user defined alert no.3']
      rim:
        description:
          - Route Injection Mechanism settings.
        type: dict
        suboptions:
          enabled:
            description:
              - Indicates whether Route Injection Mechanism is enabled.
            type: bool
          enable_on_gateways:
            description:
              - Indicates whether to enable automatic Route Injection Mechanism for gateways.
            type: bool
          route_injection_track:
            description:
              - Route injection track method.
            type: str
            choices: ['none', 'log', 'popup alert', 'mail alert', 'snmp trap alert', 'user defined alert no.1', 'user defined alert no.2',
                     'user defined alert no.3']
      tunnel_down_track:
        description:
          - VPN community permanent tunnels down track option.
        type: str
        choices: ['none', 'log', 'popup alert', 'mail alert', 'snmp trap alert', 'user defined alert no.1', 'user defined alert no.2',
                 'user defined alert no.3']
      tunnel_up_track:
        description:
          - Permanent tunnels up track option.
        type: str
        choices: ['none', 'log', 'popup alert', 'mail alert', 'snmp trap alert', 'user defined alert no.1', 'user defined alert no.2',
                 'user defined alert no.3']
  shared_secrets:
    description:
      - Shared secrets for external gateways.
      - Available from R80.10 management version.
    type: list
    elements: dict
    suboptions:
      external_gateway:
        description:
          - External gateway identified by the name or UID.
        type: str
      shared_secret:
        description:
          - Shared secret.
        type: str
  tags:
    description:
      - Collection of tag identifiers.
    type: list
    elements: str
  use_shared_secret:
    description:
      - Indicates whether the shared secret should be used for all external gateways.
      - Available from R80.10 management version.
    type: bool
  wire_mode:
    description:
      - VPN Community Wire mode properties.
      - Available from R82 JHF management version.
    type: dict
    version_added: "6.5.0"
    suboptions:
      allow_uninspected_encrypted_traffic:
        description:
          - Allow uninspected encrypted traffic between Wire mode interfaces of this Community members.
        type: bool
      allow_uninspected_encrypted_routing:
        description:
          - Allow members to route uninspected encrypted traffic in VPN routing configurations.
        type: bool
  routing_mode:
    description:
      - VPN Community Routing Mode.
      - Available from R82 JHF management version.
    type: str
    choices: ['domain_based', 'route_based']
    version_added: "6.5.0"
  advanced_properties:
    description:
      - Advanced properties.
      - Available from R82 JHF management version.
    type: dict
    version_added: "6.5.0"
    suboptions:
      support_ip_compression:
        description:
          - Indicates whether to support IP compression.
        type: bool
      use_aggressive_mode:
        description:
          - Indicates whether to use aggressive mode.
        type: bool
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-vpn-community-meshed
  cp_mgmt_vpn_community_meshed:
    encryption_method: prefer ikev2 but support ikev1
    encryption_suite: custom
    ike_phase_1:
      data_integrity: sha1
      diffie_hellman_group: group 19
      encryption_algorithm: aes-128
    ike_phase_2:
      data_integrity: aes-xcbc
      encryption_algorithm: aes-gcm-128
    name: New_VPN_Community_Meshed_1
    state: present

- name: set-vpn-community-meshed
  cp_mgmt_vpn_community_meshed:
    encryption_method: ikev2 only
    encryption_suite: custom
    ike_phase_1:
      data_integrity: sha1
      diffie_hellman_group: group 19
      encryption_algorithm: aes-128
    ike_phase_2:
      data_integrity: aes-xcbc
      encryption_algorithm: aes-gcm-128
    name: New_VPN_Community_Meshed_1
    state: present

- name: delete-vpn-community-meshed
  cp_mgmt_vpn_community_meshed:
    name: New_VPN_Community_Meshed_1
    state: absent
"""

RETURN = """
cp_mgmt_vpn_community_meshed:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_objects,
    api_call,
)


def main():
    argument_spec = dict(
        name=dict(type="str", required=True),
        disable_nat=dict(type="bool"),
        encrypted_traffic=dict(
            type="dict", options=dict(
                enabled=dict(type="bool")
            ),
        ),

        encryption_method=dict(
            type="str",
            choices=[
                "prefer ikev2 but support ikev1",
                "ikev2 only",
                "ikev1 for ipv4 and ikev2 for ipv6 only",
            ],
        ),
        encryption_suite=dict(
            type="str",
            choices=[
                "suite-b-gcm-256",
                "custom",
                "vpn b",
                "vpn a",
                "suite-b-gcm-128",
            ],
        ),
        excluded_services=dict(type="list", elements="str"),
        gateways=dict(type="list", elements="str"),
        ike_phase_1=dict(
            type="dict",
            options=dict(
                data_integrity=dict(
                    type="str",
                    choices=["aes-xcbc", "sha1", "sha256", "sha384", "sha512", "md5"],
                ),
                diffie_hellman_group=dict(
                    type="str",
                    choices=[
                        "group-1",
                        "group-2",
                        "group-5",
                        "group-14",
                        "group-19",
                        "group-20",
                    ],
                ),
                encryption_algorithm=dict(
                    type="str",
                    choices=["cast", "aes-256", "des", "aes-128", "3des"],
                ),
            ),
        ),
        ike_phase_2=dict(
            type="dict",
            options=dict(
                data_integrity=dict(
                    type="str",
                    choices=["aes-xcbc", "sha1", "sha256", "sha384", "sha512", "md5"],
                ),
                encryption_algorithm=dict(
                    type="str",
                    choices=[
                        "cast",
                        "aes-gcm-256",
                        "cast-40",
                        "aes-256",
                        "des",
                        "aes-128",
                        "3des",
                        "des-40cp",
                        "aes-gcm-128",
                        "none",
                    ],
                ),
            ),
        ),
        permanent_tunnels=dict(
            type="dict",
            options=dict(
                set_permanent_tunnels=dict(
                    type="str",
                    choices=[
                        "on all tunnels in the community",
                        "on all tunnels of specific gateways",
                        "on specific tunnels in the community",
                        "off",
                    ],
                ),
                gateways=dict(
                    type="list",
                    elements="dict",
                    options=dict(
                        gateway=dict(type="str"),
                        track_options=dict(
                            type="str",
                            choices=[
                                "according to community track options",
                                "override track options",
                            ],
                        ),
                        override_tunnel_down_track=dict(
                            type="str",
                            choices=[
                                "none",
                                "log",
                                "popup alert",
                                "mail alert",
                                "snmp trap alert",
                                "user defined alert no.1",
                                "user defined alert no.2",
                                "user defined alert no.3",
                            ],
                        ),
                        override_tunnel_up_track=dict(
                            type="str",
                            choices=[
                                "none",
                                "log",
                                "popup alert",
                                "mail alert",
                                "snmp trap alert",
                                "user defined alert no.1",
                                "user defined alert no.2",
                                "user defined alert no.3",
                            ],
                        ),
                    ),
                ),
                tunnels=dict(
                    type="list",
                    elements="dict",
                    options=dict(
                        first_tunnel_endpoint=dict(type="str"),
                        second_tunnel_endpoint=dict(type="str"),
                        track_options=dict(
                            type="str",
                            choices=[
                                "according to community track options",
                                "override track options",
                            ],
                        ),
                        override_tunnel_down_track=dict(
                            type="str",
                            choices=[
                                "none",
                                "log",
                                "popup alert",
                                "mail alert",
                                "snmp trap alert",
                                "user defined alert no.1",
                                "user defined alert no.2",
                                "user defined alert no.3",
                            ],
                        ),
                        override_tunnel_up_track=dict(
                            type="str",
                            choices=[
                                "none",
                                "log",
                                "popup alert",
                                "mail alert",
                                "snmp trap alert",
                                "user defined alert no.1",
                                "user defined alert no.2",
                                "user defined alert no.3",
                            ],
                        ),
                    ),
                ),
                rim=dict(
                    type="dict",
                    options=dict(
                        enabled=dict(type="bool"),
                        enable_on_gateways=dict(type="bool"),
                        route_injection_track=dict(
                            type="str",
                            choices=[
                                "none",
                                "log",
                                "popup alert",
                                "mail alert",
                                "snmp trap alert",
                                "user defined alert no.1",
                                "user defined alert no.2",
                                "user defined alert no.3",
                            ],
                        ),
                    ),
                ),
                tunnel_down_track=dict(
                    type="str",
                    choices=[
                        "none",
                        "log",
                        "popup alert",
                        "mail alert",
                        "snmp trap alert",
                        "user defined alert no.1",
                        "user defined alert no.2",
                        "user defined alert no.3",
                    ],
                ),
                tunnel_up_track=dict(
                    type="str",
                    choices=[
                        "none",
                        "log",
                        "popup alert",
                        "mail alert",
                        "snmp trap alert",
                        "user defined alert no.1",
                        "user defined alert no.2",
                        "user defined alert no.3",
                    ],
                ),
            ),
        ),
        shared_secrets=dict(
            type="list",
            elements="dict",
            no_log=True,
            options=dict(
                external_gateway=dict(type="str"),
                shared_secret=dict(type="str", no_log=True),
            ),
        ),
        tags=dict(type="list", elements="str"),
        use_shared_secret=dict(type="bool"),
        wire_mode=dict(
            type="dict",
            options=dict(
                allow_uninspected_encrypted_traffic=dict(type="bool"),
                allow_uninspected_encrypted_routing=dict(type="bool"),
            ),
        ),
        routing_mode=dict(
            type="str",
            choices=["domain_based", "route_based"],
        ),
        advanced_properties=dict(
            type="dict",
            options=dict(
                support_ip_compression=dict(type="bool"),
                use_aggressive_mode=dict(type="bool"),
            ),
        ),
        color=dict(
            type="str",
            choices=[
                "aquamarine",
                "black",
                "blue",
                "crete blue",
                "burlywood",
                "cyan",
                "dark green",
                "khaki",
                "orchid",
                "dark orange",
                "dark sea green",
                "pink",
                "turquoise",
                "dark blue",
                "firebrick",
                "brown",
                "forest green",
                "gold",
                "dark gold",
                "gray",
                "dark gray",
                "light green",
                "lemon chiffon",
                "coral",
                "sea green",
                "sky blue",
                "magenta",
                "purple",
                "slate blue",
                "violet red",
                "navy blue",
                "olive",
                "orange",
                "red",
                "sienna",
                "yellow",
            ],
        ),
        comments=dict(type="str"),
        details_level=dict(type="str", choices=["uid", "standard", "full"]),
        ignore_warnings=dict(type="bool"),
        ignore_errors=dict(type="bool"),
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    api_call_object = "vpn-community-meshed"

    result = api_call(module, api_call_object)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



# (c) 2022 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
author:
  - Ansible Networking Team (@ansible-network)
name: grpc
short_description: Provides a persistent connection using the gRPC protocol
description:
  - This connection plugin provides a connection to remote devices over gRPC and
    is typically used with devices for sending and receiving RPC calls
    over gRPC framework.
  - Note this connection plugin requires the grpcio python library to be installed on the
    local Ansible controller.
version_added: "3.1.0"
requirements:
  - grpcio
  - protobuf
extends_documentation_fragment:
  - ansible.netcommon.connection_persistent
options:
  host:
    description:
      - Specifies the remote device FQDN or IP address to establish the gRPC
        connection to.
    default: inventory_hostname
    type: string
    vars:
      - name: ansible_host
  port:
    type: int
    description:
      - Specifies the port on the remote device that listens for connections
        when establishing the gRPC connection. If None only the C(host) part will
        be used.
    ini:
      - section: defaults
        key: remote_port
    env:
      - name: ANSIBLE_REMOTE_PORT
    vars:
      - name: ansible_port
  network_os:
    description:
      - Configures the device platform network operating system. This value is
        used to load a device specific grpc plugin to communicate with the remote
        device.
    type: string
    vars:
      - name: ansible_network_os
  remote_user:
    description:
      - The username used to authenticate to the remote device when the gRPC
        connection is first established.  If the remote_user is not specified,
        the connection will use the username of the logged in user.
      - Can be configured from the CLI via the C(--user) or C(-u) options.
    type: string
    ini:
      - section: defaults
        key: remote_user
    env:
      - name: ANSIBLE_REMOTE_USER
    vars:
      - name: ansible_user
  password:
    description:
      - Configures the user password used to authenticate to the remote device
        when first establishing the gRPC connection.
    type: string
    vars:
      - name: ansible_password
      - name: ansible_ssh_pass
  private_key_file:
    description:
      - The PEM encoded private key file used to authenticate to the
        remote device when first establishing the grpc connection.
    type: string
    ini:
      - section: grpc_connection
        key: private_key_file
    env:
      - name: ANSIBLE_PRIVATE_KEY_FILE
    vars:
      - name: ansible_private_key_file
  root_certificates_file:
    description:
      - The PEM encoded root certificate file used to create a SSL-enabled
        channel, if the value is None it reads the root certificates from
        a default location chosen by gRPC at runtime.
    type: string
    ini:
      - section: grpc_connection
        key: root_certificates_file
    env:
      - name: ANSIBLE_ROOT_CERTIFICATES_FILE
    vars:
      - name: ansible_root_certificates_file
  certificate_chain_file:
    description:
      - The PEM encoded certificate chain file used to create a SSL-enabled
        channel. If the value is None, no certificate chain is used.
    type: string
    ini:
      - section: grpc_connection
        key: certificate_chain_file
    env:
      - name: ANSIBLE_CERTIFICATE_CHAIN_FILE
    vars:
      - name: ansible_certificate_chain_file
  ssl_target_name_override:
    description:
      - The option overrides SSL target name used for SSL host name checking.
        The name used for SSL host name checking will be the target parameter
        (assuming that the secure channel is an SSL channel). If this parameter is
        specified and the underlying is not an SSL channel, it will just be ignored.
    type: string
    ini:
      - section: grpc_connection
        key: ssl_target_name_override
    env:
      - name: ANSIBLE_GPRC_SSL_TARGET_NAME_OVERRIDE
    vars:
      - name: ansible_grpc_ssl_target_name_override
  grpc_type:
    description:
        - This option indicates the grpc type and it can be used
          in place of network_os. (example cisco.iosxr.iosxr)
    default: False
    ini:
      - section: grpc_connection
        key: type
    env:
      - name: ANSIBLE_GRPC_CONNECTION_TYPE
    vars:
      - name: ansible_grpc_connection_type
"""

from importlib import import_module

from ansible.errors import AnsibleConnectionFailure, AnsibleError
from ansible.plugins.connection import NetworkConnectionBase


try:
    from grpc import insecure_channel, secure_channel, ssl_channel_credentials
    from grpc.beta import implementations

    HAS_GRPC = True
except ImportError:
    HAS_GRPC = False

try:
    from google import protobuf  # noqa: F401  # pylint: disable=unused-import

    HAS_PROTOBUF = True
except ImportError:
    HAS_PROTOBUF = False


class Connection(NetworkConnectionBase):
    """GRPC connections"""

    transport = "ansible.netcommon.grpc"
    has_pipelining = False

    def __init__(self, play_context, new_stdin, *args, **kwargs):
        super(Connection, self).__init__(play_context, new_stdin, *args, **kwargs)

        grpc_type = self._network_os or self.get_option("grpc_type")
        if grpc_type:
            if not HAS_PROTOBUF:
                raise AnsibleError(
                    "protobuf is required to use the grpc connection type. Please run 'pip install protobuf'"
                )
            if not self._network_os:
                self._network_os = grpc_type
            cref = dict(zip(["corg", "cname", "plugin"], grpc_type.split(".")))
            grpclib = "ansible_collections.{corg}.{cname}.plugins.sub_plugins.grpc.{plugin}".format(
                **cref
            )
            grpccls = getattr(import_module(grpclib), "Grpc")
            grpc_obj = grpccls(self)

            if grpc_obj:
                self._sub_plugin = {
                    "type": "grpc",
                    "name": grpc_type,
                    "obj": grpc_obj,
                }
                self.queue_message("log", "loaded gRPC plugin for type %s" % grpc_type)
                self.queue_message("log", "grpc type is set to %s" % grpc_type)
            else:
                raise AnsibleConnectionFailure(
                    "unable to load API plugin for network_os %s" % grpc_type
                )
        else:
            raise AnsibleConnectionFailure(
                "Unable to automatically determine gRPC implementation type."
                " Please manually configure ansible_network_os value or grpc_type configuration for this host",
            )

    def _connect(self):
        """
        Create GRPC connection to target host
        :return: None
        """
        if not HAS_GRPC:
            raise AnsibleError(
                "grpcio is required to use the gRPC connection type. Please run 'pip install grpcio'"
            )
        host = self.get_option("host")
        host = self._play_context.remote_addr
        if self.connected:
            self.queue_message("log", "gRPC connection to host %s already exist" % host)
            return

        port = self.get_option("port")
        self._target = host if port is None else "%s:%d" % (host, port)
        self._timeout = self.get_option("persistent_command_timeout")
        self._login_credentials = [
            ("username", self.get_option("remote_user")),
            ("password", self.get_option("password")),
        ]
        ssl_target_name_override = self.get_option("ssl_target_name_override")
        if ssl_target_name_override:
            self._channel_options = [
                ("grpc.ssl_target_name_override", ssl_target_name_override),
            ]
        else:
            self._channel_options = None

        certs = {}
        private_key_file = self.get_option("private_key_file")
        root_certificates_file = self.get_option("root_certificates_file")
        certificate_chain_file = self.get_option("certificate_chain_file")

        try:
            if root_certificates_file:
                with open(root_certificates_file, "rb") as f:
                    certs["root_certificates"] = f.read()
            if private_key_file:
                with open(private_key_file, "rb") as f:
                    certs["private_key"] = f.read()
            if certificate_chain_file:
                with open(certificate_chain_file, "rb") as f:
                    certs["certificate_chain"] = f.read()
        except Exception as e:
            raise AnsibleConnectionFailure("Failed to read certificate keys: %s" % e)
        if certs:
            creds = ssl_channel_credentials(**certs)
            channel = secure_channel(self._target, creds, options=self._channel_options)
        else:
            channel = insecure_channel(self._target, options=self._channel_options)

        self.queue_message(
            "vvv",
            "ESTABLISH GRPC CONNECTION FOR USER: %s on PORT %s TO %s"
            % (self.get_option("remote_user"), port, host),
        )
        self._channel = implementations.Channel(channel)
        self.queue_message("vvvv", "grpc connection has completed successfully")
        self._connected = True

    def close(self):
        """
        Close the active session to the device
        :return: None
        """
        if self._connected:
            self.queue_message("vvvv", "closing gRPC connection to target host")
            self._channel.close()
        super(Connection, self).close()



#!/usr/bin/python
#
# Copyright (c) 2019 Zim Kalinowski, (@zikalino)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_devtestlabschedule
version_added: "0.1.2"
short_description: Manage Azure DevTest Lab Schedule instance
description:
    - Create, update and delete instance of Azure DecTest Lab Schedule.

options:
    resource_group:
        description:
            - The name of the resource group.
        required: True
        type: str
    lab_name:
        description:
            - The name of the lab.
        required: True
        type: str
    name:
        description:
            - The name of the schedule.
        required: True
        type: str
        choices:
            - lab_vms_startup
            - lab_vms_shutdown
    time:
        description:
            - The time of day the schedule will occur.
        type: str
    time_zone_id:
        description:
            - The time zone ID.
        type: str
    state:
        description:
            - Assert the state of the Schedule.
            - Use C(present) to create or update an Schedule and C(absent) to delete it.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Zim Kalinowski (@zikalino)

'''

EXAMPLES = '''
- name: Create (or update) DevTest Lab Schedule
  azure_rm_devtestlabschedule:
    resource_group: myResourceGroup
    lab_name: myLab
    name: lab_vms_shutdown
    time: "1030"
    time_zone_id: "UTC+12"
'''

RETURN = '''
id:
    description:
        - The identifier of the resource.
    returned: always
    type: str
    sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourcegroups/myResourceGroup/providers/microsoft.devtestlab/labs/myLab/schedules/l
             abVmsShutdown"
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.polling import LROPoller
    from azure.core.exceptions import ResourceNotFoundError
    from azure.mgmt.devtestlabs import DevTestLabsClient
except ImportError:
    # This is handled in azure_rm_common
    pass


class Actions:
    NoAction, Create, Update, Delete = range(4)


class AzureRMSchedule(AzureRMModuleBase):
    """Configuration class for an Azure RM Schedule resource"""

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            lab_name=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True,
                choices=['lab_vms_startup', 'lab_vms_shutdown']
            ),
            time=dict(
                type='str'
            ),
            time_zone_id=dict(
                type='str'
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        self.resource_group = None
        self.lab_name = None
        self.name = None
        self.schedule = dict()

        self.results = dict(changed=False)
        self.mgmt_client = None
        self.state = None
        self.to_do = Actions.NoAction

        required_if = [
            ('state', 'present', ['time', 'time_zone_id'])
        ]

        super(AzureRMSchedule, self).__init__(derived_arg_spec=self.module_arg_spec,
                                              supports_check_mode=True,
                                              supports_tags=True,
                                              required_if=required_if)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                self.schedule[key] = kwargs[key]

        self.schedule['status'] = "Enabled"

        if self.name == 'lab_vms_startup':
            self.name = 'LabVmsStartup'
            self.schedule['task_type'] = 'LabVmsStartupTask'
        elif self.name == 'lab_vms_shutdown':
            self.name = 'LabVmsShutdown'
            self.schedule['task_type'] = 'LabVmsShutdownTask'

        if self.state == 'present':
            self.schedule['daily_recurrence'] = {'time': self.schedule.pop('time')}
            self.schedule['time_zone_id'] = self.schedule['time_zone_id'].upper()

        response = None

        self.mgmt_client = self.get_mgmt_svc_client(DevTestLabsClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager)

        resource_group = self.get_resource_group(self.resource_group)

        old_response = self.get_schedule()

        if not old_response:
            self.log("Schedule instance doesn't exist")
            if self.state == 'absent':
                self.log("Old instance didn't exist")
            else:
                self.to_do = Actions.Create
        else:
            self.log("Schedule instance already exists")
            if self.state == 'absent':
                self.to_do = Actions.Delete
            elif self.state == 'present':
                if (not default_compare(self.schedule, old_response, '', self.results)):
                    self.to_do = Actions.Update

        if (self.to_do == Actions.Create) or (self.to_do == Actions.Update):
            self.log("Need to Create / Update the Schedule instance")

            if self.check_mode:
                self.results['changed'] = True
                return self.results

            response = self.create_update_schedule()

            self.results['changed'] = True
            self.log("Creation / Update done")
        elif self.to_do == Actions.Delete:
            self.log("Schedule instance deleted")
            self.results['changed'] = True

            if self.check_mode:
                return self.results

            self.delete_schedule()
            # This currently doesn't work as there is a bug in SDK / Service
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
        else:
            self.log("Schedule instance unchanged")
            self.results['changed'] = False
            response = old_response

        if self.state == 'present':
            self.results.update({
                'id': response.get('id', None)
            })
        return self.results

    def create_update_schedule(self):
        '''
        Creates or updates Schedule with the specified configuration.

        :return: deserialized Schedule instance state dictionary
        '''
        self.log("Creating / Updating the Schedule instance {0}".format(self.name))

        try:
            response = self.mgmt_client.schedules.create_or_update(resource_group_name=self.resource_group,
                                                                   lab_name=self.lab_name,
                                                                   name=self.name,
                                                                   schedule=self.schedule)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)

        except Exception as exc:
            self.log('Error attempting to create the Schedule instance.')
            self.fail("Error creating the Schedule instance: {0}".format(str(exc)))
        return response.as_dict()

    def delete_schedule(self):
        '''
        Deletes specified Schedule instance in the specified subscription and resource group.

        :return: True
        '''
        self.log("Deleting the Schedule instance {0}".format(self.name))
        try:
            response = self.mgmt_client.schedules.delete(resource_group_name=self.resource_group,
                                                         lab_name=self.lab_name,
                                                         name=self.name)
        except Exception as e:
            self.log('Error attempting to delete the Schedule instance.')
            self.fail("Error deleting the Schedule instance: {0}".format(str(e)))

        return True

    def get_schedule(self):
        '''
        Gets the properties of the specified Schedule.

        :return: deserialized Schedule instance state dictionary
        '''
        self.log("Checking if the Schedule instance {0} is present".format(self.name))
        found = False
        try:
            response = self.mgmt_client.schedules.get(resource_group_name=self.resource_group,
                                                      lab_name=self.lab_name,
                                                      name=self.name)
            found = True
            self.log("Response : {0}".format(response))
            self.log("Schedule instance : {0} found".format(response.name))
        except ResourceNotFoundError as e:
            self.log('Did not find the Schedule instance.')
        if found is True:
            return response.as_dict()

        return False


def default_compare(new, old, path, result):
    if new is None:
        return True
    elif isinstance(new, dict):
        if not isinstance(old, dict):
            result['compare'] = 'changed [' + path + '] old dict is null'
            return False
        for k in new.keys():
            if not default_compare(new.get(k), old.get(k, None), path + '/' + k, result):
                return False
        return True
    elif isinstance(new, list):
        if not isinstance(old, list) or len(new) != len(old):
            result['compare'] = 'changed [' + path + '] length is different or null'
            return False
        if isinstance(old[0], dict):
            key = None
            if 'id' in old[0] and 'id' in new[0]:
                key = 'id'
            elif 'name' in old[0] and 'name' in new[0]:
                key = 'name'
            else:
                key = list(old[0])[0]
            new = sorted(new, key=lambda x: x.get(key, None))
            old = sorted(old, key=lambda x: x.get(key, None))
        else:
            new = sorted(new)
            old = sorted(old)
        for i in range(len(new)):
            if not default_compare(new[i], old[i], path + '/*', result):
                return False
        return True
    else:
        if path == '/location':
            new = new.replace(' ', '').lower()
            old = new.replace(' ', '').lower()
        if new == old:
            return True
        else:
            result['compare'] = 'changed [' + path + '] ' + str(new) + ' != ' + str(old)
            return False


def main():
    """Main execution"""
    AzureRMSchedule()


if __name__ == '__main__':
    main()
